#$Id$#

from books.model.Contact import Contact
from books.model.Email import Email
from books.model.Address import Address
from books.model.ContactPerson import ContactPerson
from books.model.ContactPersonList import ContactPersonList
from books.model.CustomField import CustomField
from books.model.DefaultTemplate import DefaultTemplate
from books.model.PageContext import PageContext
from books.model.ContactList import ContactList
from books.model.ToContact import ToContact
from books.model.FromEmail import FromEmail
from books.model.CommentList import CommentList
from books.model.Comment import Comment
from books.model.CreditNoteRefundList import CreditNoteRefundList
from books.model.CreditNoteRefund import CreditNoteRefund

class ContactParser:
    """This class is used to parse the json for contact."""

    def get_contact(self, resp):
        """This method parses the given response and creates a contact object.

        Args:
            resp(dict): Response containing json object for contact.

        Returns:
            instance: Contact object.
 
        """
        contact = Contact()
        response = resp['contact']
        contact.set_contact_id(response['contact_id'])
        contact.set_contact_name(response['contact_name'])
        contact.set_company_name(response['company_name'])
        contact.set_contact_salutation(response['contact_salutation'])
        contact.set_price_precision(response['price_precision'])
        contact.set_has_transaction(response['has_transaction'])
        contact.set_contact_type(response['contact_type'])
        contact.set_is_crm_customer(response['is_crm_customer'])
        contact.set_primary_contact_id(response['primary_contact_id'])
        contact.set_payment_terms(response['payment_terms'])
        contact.set_payment_terms_label(response['payment_terms_label'])
        contact.set_currency_id(response['currency_id'])
        contact.set_currency_code(response['currency_code'])
        contact.set_currency_symbol(response['currency_symbol'])
        contact.set_gst_treatment(response['gst_treatment'])
        contact.set_gst_no(response['gst_no'])
        contact.set_place_of_contact(response['place_of_contact'])
        contact.set_outstanding_receivable_amount(response[\
            'outstanding_receivable_amount'])
        contact.set_outstanding_receivable_amount_bcy(response[\
            'outstanding_receivable_amount_bcy'])
        contact.set_outstanding_payable_amount(response[\
            'outstanding_payable_amount'])
        contact.set_outstanding_payable_amount_bcy(response[\
            'outstanding_payable_amount_bcy'])
        contact.set_unused_credits_receivable_amount(response[\
            'unused_credits_receivable_amount'])
        contact.set_unused_credits_receivable_amount_bcy(response[\
            'unused_credits_receivable_amount_bcy'])
        contact.set_unused_credits_payable_amount(response[\
            'unused_credits_payable_amount'])
        contact.set_unused_credits_payable_amount_bcy(response[\
            'unused_credits_payable_amount_bcy'])
        contact.set_status(response['status'])
        contact.set_payment_reminder_enabled(response[\
            'payment_reminder_enabled'])

        for value in response['custom_fields']:
            custom_field = CustomField()
            custom_field.set_index(value['index'])
            custom_field.set_value(value['value'])
            custom_field.set_label(value['label'])
            contact.set_custom_fields(custom_field)
    
        billing_address = Address()
        billing_address.set_address(response['billing_address']['address'])
        billing_address.set_city(response['billing_address']['city'])
        billing_address.set_state(response['billing_address']['state'])
        billing_address.set_zip(response['billing_address']['zip'])
        billing_address.set_country(response['billing_address']['country'])
        billing_address.set_fax(response['billing_address']['fax'])
        contact.set_billing_address(billing_address)
    
        shipping_address = Address()
        shipping_address.set_address(response['shipping_address']['address'])
        shipping_address.set_city(response['shipping_address']['city'])
        shipping_address.set_state(response['shipping_address']['state'])
        shipping_address.set_zip(response['shipping_address']['zip'])
        shipping_address.set_country(response['shipping_address']['country'])
        shipping_address.set_fax(response['shipping_address']['fax'])
        contact.set_shipping_address(shipping_address)

        contact_persons = []
        for value in response['contact_persons']:
            contact_person = ContactPerson()
            contact_person.set_contact_person_id(value['contact_person_id'])
            contact_person.set_salutation(value['salutation'])
            contact_person.set_first_name(value['first_name'])
            contact_person.set_last_name(value['last_name'])
            contact_person.set_email(value['email'])
            contact_person.set_phone(value['phone'])
            contact_person.set_mobile(value['mobile'])
            contact_person.set_is_primary_contact(value['is_primary_contact'])
            contact_persons.append(contact_person)
        contact.set_contact_persons(contact_persons)
  
        default_templates = DefaultTemplate()
        default_templates.set_invoice_template_id(response[
            'default_templates']['invoice_template_id'])
        default_templates.set_invoice_template_name(response[\
            'default_templates']['invoice_template_name'])
        default_templates.set_estimate_template_id(response[\
            'default_templates']['estimate_template_id'])
        default_templates.set_estimate_template_name(response[\
            'default_templates']['estimate_template_name'])
        default_templates.set_creditnote_template_id(response[\
            'default_templates']['creditnote_template_id'])
        default_templates.set_creditnote_template_name(response[\
            'default_templates']['creditnote_template_name'])
        default_templates.set_invoice_email_template_id(response[\
            'default_templates']['invoice_email_template_id'])
        default_templates.set_invoice_email_template_name(response[\
            'default_templates']['invoice_email_template_name'])
        default_templates.set_estimate_email_template_id(response[\
            'default_templates']['estimate_email_template_id'])
        default_templates.set_estimate_email_template_name(response[\
            'default_templates']['estimate_email_template_name']) 
        default_templates.set_creditnote_email_template_id(response[\
            'default_templates']['creditnote_email_template_id'])
        default_templates.set_creditnote_email_template_name(response[\
            'default_templates']['creditnote_email_template_name'])
    
        contact.set_default_templates(default_templates)
        contact.set_notes(response['notes'])
        contact.set_created_time(response['created_time'])
        contact.set_last_modified_time(response['last_modified_time'])
        return contact
 
    def get_contacts(self, resp):
        """This method parses the given response and creates a contact 
            list object.
  
        Args:
            resp(dict): Response containing json object for contact list.

        Returns:
            instance: Contact list object.
 
        """
        contacts_list = ContactList()
        response = resp['contacts']
        for value in resp['contacts']:
            contact = Contact()
            contact.set_contact_id(value['contact_id'])
            contact.set_contact_name(value['contact_name'])
            contact.set_company_name(value['company_name'])
            contact.set_gst_treatment(value['gst_treatment'])
            contact.set_gst_no(value['gst_no'])
            contact.set_place_of_contact(value['place_of_contact'])
            contact.set_contact_type(value['contact_type'])
            contact.set_status(value['status'])
            contact.set_payment_terms(value['payment_terms'])
            contact.set_payment_terms_label(value['payment_terms_label'])
            contact.set_currency_id(value['currency_id'])
            contact.set_currency_code(value['currency_code'])
            contact.set_outstanding_receivable_amount(value[\
                'outstanding_receivable_amount'])
            contact.set_outstanding_payable_amount(value[\
                'outstanding_payable_amount'])
            contact.set_unused_credits_receivable_amount(value[\
                'unused_credits_receivable_amount'])
            contact.set_unused_credits_payable_amount(value[\
                'unused_credits_payable_amount'])
            contact.set_first_name(value['first_name'])
            contact.set_last_name(value['last_name'])
            contact.set_email(value['email'])
            contact.set_phone(value['phone'])
            contact.set_mobile(value['mobile'])
            contact.set_created_time(value['created_time'])
            contact.set_last_modified_time(value['last_modified_time'])
            contacts_list.set_contacts(contact)
        page_context_object = PageContext()
        page_context = resp['page_context']
        page_context_object.set_page(page_context['page'])
        page_context_object.set_per_page(page_context['per_page'])
        page_context_object.set_has_more_page(page_context['has_more_page'])
        page_context_object.set_applied_filter(page_context['applied_filter'])
        page_context_object.set_sort_column(page_context['sort_column']) 
        page_context_object.set_sort_order(page_context['sort_order']) 
    
        contacts_list.set_page_context(page_context_object)
        return contacts_list

    def get_contact_person(self, resp):
        """This method parses the given response and creates a contact person 
            object.
 
        Args:
            resp(dict): Response containing json object for contact person.

        Returns:
            instance: Contact person object.

        """
        contact_person = ContactPerson() 
        response = resp['contact_person']
        contact_person.set_contact_id(response['contact_id'])
        contact_person.set_contact_person_id(response['contact_person_id'])
        contact_person.set_salutation(response['salutation'])
        contact_person.set_first_name(response['first_name'])
        contact_person.set_last_name(response['last_name'])
        contact_person.set_email(response['email'])
        contact_person.set_phone(response['phone'])
        contact_person.set_mobile(response['mobile'])
        contact_person.set_is_primary_contact(response['is_primary_contact'])
     
        return contact_person

    def get_contact_persons(self, response):
        """This method parses the given repsonse and creates contact persons 
            list object.

        Args: 
            response(dict): Response containing json object for contact 
                                persons list.

        Returns: 
            instance: Contact person list object.

        """
        resp = response['contact_persons']
        contact_person_list = ContactPersonList()
        for value in resp:
            contact_person = ContactPerson()
            contact_person.set_contact_person_id(value['contact_person_id'])
            contact_person.set_salutation(value['salutation'])
            contact_person.set_first_name(value['first_name'])
            contact_person.set_last_name(value['last_name'])
            contact_person.set_email(value['email'])
            contact_person.set_phone(value['phone'])
            contact_person.set_mobile(value['mobile'])
            contact_person.set_is_primary_contact(value['is_primary_contact'])    
            contact_person_list.set_contact_persons(contact_person)
        page_context_object = PageContext()
        page_context = response['page_context']
        page_context_object.set_page(page_context['page'])
        page_context_object.set_per_page(page_context['per_page'])
        page_context_object.set_has_more_page(page_context['has_more_page'])
        page_context_object.set_sort_column(page_context['sort_column'])
        page_context_object.set_sort_order(page_context['sort_order'])
    
        contact_person_list.set_page_context(page_context_object)
        return contact_person_list


    def get_message(self, response):
        """This method parses the given response and returns the message.

        Args: 
            response(dict): Response containing json object.

        Returns: 
            str: Success message.

        """
        return response['message']

    def get_mail_content(self, response):
        """This method parses the give response and creates object for email.

        Args:
            response(dict): Response containing json object for email .

        Returns:
            instance: Email object.

        """   
        email = Email()
        data = response['data']
        email.set_body(data['body'])
        email.set_contact_id(data['contact_id'])
        email.set_subject(data['subject'])
        for to_contact in data['to_contacts']:
            to_contacts = ToContact()
            to_contacts.set_first_name(to_contact['first_name'])
            to_contacts.set_selected(to_contact['selected'])
            to_contacts.set_phone(to_contact['phone'])
            to_contacts.set_email(to_contact['email'])
            to_contacts.set_last_name(to_contact['last_name'])
            to_contacts.set_salutation(to_contact['salutation'])
            to_contacts.set_contact_person_id(to_contact['contact_person_id'])
            to_contacts.set_mobile(to_contact['mobile'])
            email.set_to_contacts(to_contacts)
        email.set_file_name(data['file_name'])
        for value in data['from_emails']:
            from_emails = FromEmail()
            from_emails.set_user_name(value['user_name'])
            from_emails.set_selected(value['selected'])
            from_emails.set_email(value['email'])
            from_emails.set_is_org_email_id(value['is_org_email_id'])
            email.set_from_emails(from_emails)

        return email

    def get_comment_list(self, response):
        """This method parses the given response and creates object for 
            comments list.

        Args:
            response(dict): Response containing json object for comments list.

        Returns:
            instance: cComments list object.

        """
        comment_list = CommentList()
        contact_comments = response['contact_comments']
        for value in contact_comments:
            contact_comment = Comment() 
            contact_comment.set_comment_id(value['comment_id'])
            contact_comment.set_contact_id(value['contact_id'])
            contact_comment.set_contact_name(value['contact_name'])
            contact_comment.set_description(value['description'])
            contact_comment.set_commented_by_id(value['commented_by_id'])
            contact_comment.set_commented_by(value['commented_by'])
            contact_comment.set_date(value['date'])
            contact_comment.set_date_description(value['date_description'])
            contact_comment.set_time(value['time'])
            contact_comment.set_transaction_id(value['transaction_id'])
            contact_comment.set_transaction_type(value['transaction_type'])
            contact_comment.set_is_entity_deleted(value['is_entity_deleted'])
            contact_comment.set_operation_type(value['operation_type'])
            comment_list.set_comments(contact_comment)
        page_context = response['page_context']
        page_context_object = PageContext()
        page_context_object.set_page(page_context['page'])
        page_context_object.set_per_page(page_context['per_page'])
        page_context_object.set_has_more_page(page_context['has_more_page'])
        page_context_object.set_applied_filter(page_context['applied_filter'])
        page_context_object.set_sort_column(page_context['sort_column'])
        page_context_object.set_sort_order(page_context['sort_order'])
        comment_list.set_page_context(page_context_object)
    
        return comment_list

    def get_refund_list(self, response):
        """This method parses the given response and creates refund list 
            object.

        Args:
            response(dict): Response containing json object for refund list.

        Returns:
            instance: Refund list object.

        """
        list_refund = CreditNoteRefundList()
        creditnote_refunds = response['creditnote_refunds']
        for value in creditnote_refunds:
            creditnote_refund = CreditNoteRefund()
            creditnote_refund.set_creditnote_refund_id(value[\
                'creditnote_refund_id'])
            creditnote_refund.set_creditnote_id(value['creditnote_id'])
            creditnote_refund.set_date(value['date'])
            creditnote_refund.set_refund_mode(value['refund_mode'])
            creditnote_refund.set_reference_number(value['reference_number'])
            creditnote_refund.set_creditnote_number(value['creditnote_number'])
            creditnote_refund.set_customer_name(value['customer_name'])
            creditnote_refund.set_description(value['description'])
            creditnote_refund.set_amount_bcy(value['amount_bcy'])
            creditnote_refund.set_amount_fcy(value['amount_fcy'])
            list_refund.set_creditnote_refunds(creditnote_refund)
        page_context = response['page_context']
        page_context_object = PageContext()
        page_context_object.set_page(page_context['page'])
        page_context_object.set_per_page(page_context['per_page'])
        page_context_object.set_has_more_page(page_context['has_more_page'])
        page_context_object.set_report_name(page_context['report_name'])
        page_context_object.set_sort_column(page_context['sort_column'])
        page_context_object.set_sort_order(page_context['sort_order'])    
        list_refund.set_page_context(page_context_object)

        return list_refund

  



#$Id$

class Item:
    """This class is used to create object for item."""
    def __init__(self):
        """Initialize parameters for Item object."""
        self.item_id = ''
        self.name = ''
        self.status = ''
        self.description = ''
        self.rate = 0.0
        self.unit = ''
        self.account_id = ''
        self.account_name = ''
        self.tax_id = ''
        self.tax_name = ''
        self.tax_percentage = 0.0
        self.tax_type = ''
        self.item_tax_preferences = []

    def set_item_tax_preferences(self, item_tax_preference):
        """Set item_tax_preferences.

        Args:
            item_tax_preference(instance): item_tax_preference object.

        """
        self.item_tax_preferences.append(item_tax_preference)

    def get_item_tax_preferences(self):
        """Get item_tax_preference.

        Returns: 
            instance: item_tax_preference object.

        """
        return self.item_tax_preferences

    def set_item_id(self, item_id):
        """Set item id.

        Args:
            item_id(str): Item id.

        """
        self.item_id = item_id

    def get_item_id(self):
        """Get item id.

        Returns:
            str: Item id.

        """
        return self.item_id

    def set_name(self, name):
        """Set name.

        Args:
            name(str): Name.

        """
        self.name = name

    def get_name(self):
        """Get name.

        Returns:
            str: Name.

        """
        return self.name

    def set_status(self, status):
        """Set status.

        Args:
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.
  
        Returns:
            str: Status.

        """
        return self.status

    def set_description(self,description):
        """Set description.
 
        Args:
            descritpion(str): Description.

        """
        self.description = description

    def get_description(self):
        """Get description.

        Returns:
            str: Descritpion.

        """
        return self.description

    def set_rate(self, rate):
        """Set rate.

        Args:
            rate(float): Rate.

        """
        self.rate = rate

    def get_rate(self):
        """Get rate.

        Returns:
            float: Rate.

        """
        return self.rate

    def set_unit(self, unit):
        """Set unit.

        Args:
            unit(float): Unit.
    
        """
        self.unit = unit

    def get_unit(self):
        """Get unit.

        Returns:
            float: Unit.

        """
        return self.unit

    def set_account_id(self, account_id):
        """Set account id.

        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self):
        """Get account id.
      
        Returns:
            str: Account id.

        """
        return self.account_id

    def set_account_name(self, account_name):
        """Set account name.

        Args:
            str: Account name.

        """
        self.account_name = account_name

    def get_account_name(self):
        """Get account name.

        Returns:
            str: Account name.

        """
        return self.account_name

    def set_tax_id(self, tax_id):
        """Set tax id.

        Args:
            tax_id(str): Tax id.

        """
        self.tax_id = tax_id

    def get_tax_id(self):
        """Get tax id.

        Returns:
            str: Tax id.

        """
        return self.tax_id

    def set_tax_name(self, tax_name):
        """Set tax name.

        Args:
            tax_name(str): Tax name.

        """
        self.tax_name = tax_name

    def get_tax_name(self):
        """Get tax name.

        Returns:
            str: Tax name.

        """
        return self.tax_name

    def set_tax_percentage(self, tax_percentage):
        """Set tax percentage.

        Args:
            tax_percentage(float): Tax percentage.

        """
        self.tax_percentage = tax_percentage

    def get_tax_percentage(self):
        """Get tax percentage.

        Returns: 
            float: Tax percentage.

        """
        return self.tax_percentage

    def set_tax_type(self, tax_type):
        """Set tax type.

        Args:
            tax_type(str): Tax type.

        """
        self.tax_type = tax_type

    def get_tax_type(self):
        """Get tax type.

        Returns:
            str: Tax type.

        """
        return self.tax_type

    def to_json(self):
        """This method is used to create json object for items.

        Returns: 
            dict: Dictionary containing json object for items.

        """
        data = {}
        if self.name != '':
            data['name'] = self.name
        if self.description != '':
            data['description'] = self.description
        if self.rate > 0:
            data['rate'] = self.rate
        if self.account_id != '':
            data['account_id'] = self.account_id 
        if self.tax_id != '':
            data['tax_id'] = self.tax_id
        if self.item_tax_preferences:
            data['item_tax_preferences'] = self.item_tax_preferences
        return data




#$Id$

from books.model.Address import Address

class CreditNote:
    """This class creates object for credit notes."""
    def __init__(self):
        """This class is used to create object for credit notes."""
        self.creditnote_id = ''
        self.creditnote_number = ''
        self.date = ''
        self.status = ''
        self.reference_number = ''
        self.customer_id = ''
        self.customer_name = ''
        self.contact_persons = []
        self.currency_id = ''
        self.currency_code = ''
        self.exchange_rate = 0.0
        self.price_precision = 0
        self.template_id = ''
        self.template_name = ''
        self.is_emailed = True
        self.line_items = []
        self.sub_total = 0.0
        self.total = 0.0
        self.total_credits_used = 0.0
        self.total_refunded_amount = 0.0
        self.balance = 0.0
        self.taxes = []
        self.notes = ''
        self.terms = ''
        self.billing_address = Address()
        self.shipping_address = Address()
        self.created_time = ''
        self.last_modified_time = ''
        self.refund_mode = ''
        self.amount = ''
        self.from_account_id = ''
        self.description = ''
  
    def set_creditnote_id(self, creditnote_id):
        """Set creditnote id.
         
        Args:
            creditnote_id(str): Credit note id.

        """
        self.creditnote_id = creditnote_id

    def get_creditnote_id(self):
        """Get creditnote id.

        Returns:
            str: Creditnote id.

        """
        return self.creditnote_id

    def set_creditnote_number(self, creditnote_number):
        """Set creditnote number.

        Args:
            creditnote_number(str):Creditnote number.

        """
        self.creditnote_number = creditnote_number

    def get_creditnote_number(self):
        """Get creditnote number.
      
        Returns:
            str: Creditnote number.
      
        """
        return self.creditnote_number

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date
 
    def get_date(self):
        """Get date.

        Returns:
            str: Date.
  
        """
        return self.date
  
    def set_status(self, status):
        """Set status.

        Args: 
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.
 
        Returns:
            str: Status.

        """
        return self.status
  
    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number
 
    def set_customer_id(self, customer_id):
        """Set customer id.

        Args:
            customer_id(str): Customerid.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns:
            str: Customer id.

        """
        return self.customer_id
  
    def set_customer_name(self, customer_name):
        """Set customer name.

        Args:
            customer_name(str): Customer name.

        """
        self.customer_name = customer_name
 
    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.

        """
        return self.customer_name

    def set_contact_persons(self, contact_person):
        """Set contact person.

        Args:
            contact_person(list): Contact person.

        """
        self.contact_persons.extend(contact_person)

    def get_contact_persons(self):
        """Get contact persons.

        Returns:
            list of instance: List of contact person object.

        """
        return self.contact_persons
  
    def set_currency_id(self, currency_id):
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.
 
        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code.

        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currenccy code.

        Returns:
            str: Currecny code.

        """
        return self.currency_code

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange ratte.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate

    def set_price_precision(self, price_precision):
        """Set price precision.

        Args:
            price_precision(float): Price precision.

        """
        self.price_precision = price_precision

    def get_price_precision(self):
        """Get price precision.

        Returns:
            float: Price precision.

        """
        return self.price_precision

    def set_template_id(self, template_id):
        """Set template id.

        Args:
            template_id(str): Template id.

        """
        self.template_id = template_id

    def get_template_id(self):
        """Get template id.

        Returns:
            str: Template id.

        """
        return self.template_id
 
    def set_template_name(self, template_name):
        """Set template name.

        Args:
            template_name(str): Template name.

        """
        self.template_name = template_name
 
    def get_template_name(self):
        """Get template name.

        Returns:
            str: Template name.

        """
        return self.template_name
  
    def set_is_emailed(self, is_emailed):
        """Set whether is emailed or not.

        Args:
            is_emailed(bool): True if emailed else False.

        """
        self.is_emailed = is_emailed

    def get_is_emailed(self):
        """Get whether is emailed or not.

        Returns:
            bool: True if emailed else False.

        """
        return self.is_emailed
  
    def set_line_items(self, line_items):
        """Set line items.

        Args:
            line_items(instance): Line items object.

        """
        self.line_items.append(line_items)

    def get_line_items(self):
        """Get line items.

        Returns:
            list of instance: List of lineitems object.

        """
        return self.line_items
  
    def set_sub_total(self, sub_total):
        """Set sub total.

        Args:
            sub_total(float): Sub total.
   
        """
        self.sub_total = sub_total

    def get_sub_total(self):
        """Get sub total.

        Returns:
            float: Sub total.

        """
        return self.sub_total

    def set_total(self, total):
        """Set total.

        Args:
            total(float): Total.
 
        """
        self.total = total

    def get_total(self):
        """Get total.

        Returns:
            float: Total.

        """
        return self.total

    def set_total_credits_used(self, total_credits_used):
        """Set total credits used.
 
        Args:
            total_credits_used(float): Total credits used.

        """
        self.total_credits_used = total_credits_used

    def get_total_credits_used(self):
        """Get total credits used.

        Returns:
            float: Total credits used.
 
        """
        return self.total_credits_used

    def set_total_refunded_amount(self, total_refunded_amount):
        """Set total refunded amount.
 
        Args:
            total_refunded_amount(float): Total refunded amount.

        """
        self.total_refunded_amount = total_refunded_amount

    def get_total_refunded_amount(self):
        """Get total refunded amount.

        Returns:
            float: Total refunded amount.

        """
        return self.total_refunded_amount

    def set_balance(self, balance):
        """Set balance.

        Args:
            balance(float): Balance.
 
        """
        self.balance = balance

    def get_balance(self):
        """Get balance.

        Returns:
            float: Balance.

        """
        return self.balance

    def set_taxes(self, taxes):
        """Set taxes.

        Args:
            taxes(list of instance): List of tax object.

        """
        self.taxes.extend(taxes)

    def get_taxes(self):
        """Get taxes.
   
        Returns:
            list: List of objects.

        """
        return self.taxes

    def set_notes(self, notes):
        """Set notes.
 
        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """
        self.terms = terms
 
    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def set_shipping_address(self, address):
        """Set shipping address.

        Arg:
           address(instance): Address object.

        """
        self.shipping_address = address

    def get_shipping_address(self):
        """Get shipping address.

        Returns:
            instance: Address object.

        """
        return self.shipping_address
 
    def set_billing_address(self, address):
        """Set billing address.
 
        Args:
            address: Address object.

        """
        self.billing_address = address

    def get_billing_address(self):
        """Get billing address.

        Returns:
            instance: Address object.

        """
        return self.billing_address

    def set_created_time(self, created_time):
        """Set created time.

        Args:
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created time.

        Returns:
            str: Created ime.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.

        Args: 
            last_modified_time(str): Last modified time.


        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self):
        """Get last modified time.

        Returns:
            str: Last modifiedd time.
      
        """
        return self.last_modified_time

    def set_refund_mode(self, refund_mode):
        """Set refund mode.
 
        Args:
            refund_mode(str): Refund mode.

        """
        self.refund_mode = refund_mode

    def get_refund_mode(self):
        """Get refund mode.

        Returns:
            str: Refund mode.

        """
        return self.refund_mode

    def set_amount(self, amount):
        """Set amount.

        Args:
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self): 
        """Get amount.
 
        Returns:
            float: Amount.

        """
        return self.amount

    def set_from_account_id(self, from_account_id):
        """Set from account id.

        Args:
            from_account_id(str): From account id.

        """ 
        self.from_account_id = from_account_id

    def get_from_account_id(self):
        """Get from account id.

        Returns:
            str: From account id.

        """
        return self.from_account_id

    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self):
        """Get description.

        Returns:
            str: Description.

        """
        return self.description 

    def to_json(self):
        """This method is used to convert credit notes object to json object.

        Returns:
            dict: Dictionary containing json object for credit notes.

        """
        data = {}
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.contact_persons:
            data['contact_persons'] = self.contact_persons
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.template_id != '':
            data['template_id'] = self.template_id
        if self.date != '':
            data['date'] = self.date
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.line_items:
            data['line_items'] = []
            for value in self.line_items:
                line_item = value.to_json()
                data['line_items'].append(line_item)
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        if self.creditnote_number != '':
            data['creditnote_number'] = self.creditnote_number
        return data



#$Id$

from books.model.PageContext import PageContext

class Statement: 
    """This class is used to create object for statement."""
    def __init__(self): 
        """Initialize parameters for Statement object."""
        self.statement_id = ''
        self.from_date = ''
        self.to_date = ''
        self.source = ''
        self.transactions = []
        self.page_context = PageContext()

    def set_statement_id(self, statement_id): 
        """Set statement id.

        Args:
            statement_id(str): Statement id.
 
        """
        self.statement_id = statement_id

    def get_statement_id(self):
        """Get statement id.

        Returns:
            str: Statement id.

        """
        return self.statement_id

    def set_from_date(self, from_date):
        """Set from date.
 
        Args:
            from_date(str): From date.

        """
        self.from_date = from_date

    def get_from_date(self):
        """Get from date.

        Returns: 
            str: From date.

        """
        return self.from_date

    def set_source(self, source): 
        """Set source.

        Args:
            source(str): Source.

        """
        self.source = source

    def get_source(self):
        """Get source.

        Returns:
            str: Source.

        """
        return self.source

    def set_transactions(self, transactions):
        """Set transactions.

        Args:
            transactions(instance): Transactions object.

        """
        self.transactions.append(transactions)

    def get_transactions(self): 
        """Get transactions.

        Returns:
            list of instance: List of transactions object.

        """
        return self.transaction

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self): 
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context

    def set_to_date(self, to_date):
        """Set to date.
       
        Args:
            to_date(str): To date.

        """
        self.to_date = to_date

    def get_to_date(self):
        """Get to date.

        Returns:
            str: To date.

        """
        return self.to_date 




#$Id$

from books.model.PageContext import PageContext

class BankAccountList:
    """This class is used to create object for Bank accounts list."""
    def __init__(self):
        """Initialize the parameters for Bank accounts list."""
        self.bank_accounts = []
        self.page_context = PageContext()

    def set_bank_accounts(self, bank_accounts): 
        """Set Bank accounts.
    
        Args:
            bank_accounts(instance): Bank accounts object.

        """
        self.bank_accounts.append(bank_accounts)

    def get_bank_accounts(self):
        """Get bank accounts.

        Returns:
            list of instance: List of bank accounts object.

        """
        return self.bank_accounts

    def set_page_context(self, page_context): 
        """Set page context.

        Args:
            page_context(instance): Page context object.
         
        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page_context.
 
        Returns:
            instance: Page context object.

        """
        return self.page_context

 



#$Id$

class EmailId:
    """This class is used to create object for email ids."""
    def __init__(self):
        """Initialize parameters for email ids."""
        self.email = ''
        self.is_selected = None

    def set_email(self, email):
        """Set email.

        Args:
            email(str): Email.
        
        """
        self.email = email

    def get_email(self):
        """Get email.

        Returns:
            str: Email.

        """
        return self.email

    def set_is_selected(self, is_selected):
        """Set whether selected or not.

        Args:
            is_selected(bool): True if selected else false.

        """
        self.is_selected = is_selected

    def get_is_selected(self):
        """Get whether selected or not.

        Returns:
            bool: True if selected else false.

        """ 
        return self.is_selected




#$Id$#

from books.model.Journal import Journal
from books.model.JournalList import JournalList
from books.model.PageContext import PageContext
from books.model.LineItem import LineItem

class JournalsParser: 
    """This class is used to parse the given json for Jornals."""
   
    def get_list(self, resp): 
        """This method parses the given response for journals list.

        Args:
            journals(dict): Response containing json object for journals list.

        Returns:
            instance: Journals list object.

        """
        journals_list = JournalList()
        for value in resp['journals']:
            journal = Journal()
            journal.set_journal_id(value['journal_id'])
            journal.set_journal_date(value['journal_date'])
            journal.set_entry_number(value['entry_number'])
            journal.set_reference_number(value['reference_number'])
            journal.set_notes(value['notes'])
            journal.set_total(value['total'])
            journals_list.set_journals(journal)
        page_context = resp['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        if 'from_date' in page_context:
            page_context_obj.set_from_date(page_context['from_date'])
        if 'to_date' in page_context:
            page_context_obj.set_to_date(page_context['to_date'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        journals_list.set_page_context(page_context_obj)
        return journals_list

    def get_journal(self, resp):
        """This method parses the given response and returns journals object.

        Args:
            resp(dict): Response containing json object for journal.

        Returns:
            instance: Journal object.

        """
        journal = resp['journal']
        journal_obj = Journal()
        journal_obj.set_journal_id(journal['journal_id'])
        journal_obj.set_entry_number(journal['entry_number'])
        journal_obj.set_reference_number(journal['reference_number'])
        journal_obj.set_notes(journal['notes'])
        journal_obj.set_currency_id(journal['currency_id'])
        journal_obj.set_currency_symbol(journal['currency_symbol'])
        journal_obj.set_journal_date(journal['journal_date'])
        for value in journal['line_items']:
            line_item = LineItem()
            line_item.set_line_id(value['line_id'])
            line_item.set_account_id(value['account_id'])
            line_item.set_account_name(value['account_name'])
            line_item.set_description(value['description'])
            line_item.set_debit_or_credit(value['debit_or_credit'])
            line_item.set_tax_id(value['tax_id'])
            line_item.set_tax_name(value['tax_name'])
            line_item.set_tax_type(value['tax_type'])
            line_item.set_tax_percentage(value['tax_percentage'])
            line_item.set_amount(value['amount'])
            journal_obj.set_line_items(line_item)
        journal_obj.set_line_item_total(journal['line_item_total'])
        journal_obj.set_total(journal['total'])
        journal_obj.set_price_precision(journal['price_precision'])
        journal_obj.set_created_time(journal['created_time'])
        journal_obj.set_last_modified_time(journal['last_modified_time'])
        for value in journal['taxes']:
            tax = Tax()
            tax.set_tax_name(value['tax_name'])
            tax.set_tax_amount(value['tax_amount'])
            journal_obj.set_taxes(tax)
        return journal_obj

    def get_message(self, resp):
        """This method parses the given response and returns string message.

        Args:
            resp(dict): Response containing json object for message.

        Returns: 
            str: Success message.

        """
        return resp['message']

             



#$Id$

from books.model.PageContext import PageContext

class CreditNoteList:
    """This class is used to create object for credit notes list."""
    def __init__(self):
        """Initialize the parameters for credditnotes list."""
        self.creditnotes = []
        self.page_context = PageContext()

    def set_creditnotes(self, creditnotes):
        """Set credit notes.

        Args:
            creditnotes(list): List of credit notes.

        """ 
        self.creditnotes.extend(creditnotes)
 
    def get_creditnotes(self):
        """Get creditnotes.
    
        Returns:
            list of instance: List of credit notes.

        """
        return self.creditnotes

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context

  



#$Id$

from books.model.Address import Address

class Estimate:
    """This class is used to create object for estimates."""
    def __init__(self):
        """Initialize parameters for Estimates object."""
        self.estimate_id = ''
        self.estimate_number = ''
        self.date = ''
        self.reference_number = ''
        self.status = ''
        self.customer_id = ''
        self.customer_name = ''
        self.contact_persons = []
        self.currency_id = ''
        self.currency_code = ''
        self.exchange_rate = 0.00
        self.expiry_date = ''
        self.discount = 0.00
        self.is_discount_before_tax = None
        self.discount_type = ''
        self.line_items = []
        self.shipping_charge = 0.00
        self.adjustment = 0.00
        self.adjustment_description = ''
        self.sub_total = 0.0
        self.total = 0.0
        self.tax_total = 0.0
        self.price_precision = 0
        self.taxes = []
        self.billing_address = Address()
        self.shipping_address = Address()
        self.notes = ''
        self.terms = ''
        self.custom_fields = []
        self.template_id = ''
        self.template_name = ''
        self.created_time = ''
        self.last_modified_time = ''
        self.salesperson_id = ''
        self.salesperson_name = ''
        self.accepted_date = ''
        self.declined_date = ''
      
    def set_estimate_id(self, estimate_id):
        """Set estimate id.

        Args:
            estimate_id(str): Estimate id.
 
        """
        self.estimate_id = estimate_id

    def get_estimate_id(self):
        """Get estimate id.

        Returns:
            str: Estimate id.

        """
        return self.estimate_id
  
    def set_estimate_number(self, estimate_number):
        """Set estimate number.

        Args: 
            estimate_number(str): Estimate number.
        
        """
        self.estimate_number = estimate_number

    def get_estimate_number(self):
        """Get estimate number.

        Returns:
            str: Estimate number.
 
        """
        return self.estimate_number
  
    def set_date(self, date):
        """Set the date for estimate.

        Args:
            date(str): Date for estimate.

        """
        self.date = date

    def get_date(self):
        """Get the date of estimate.

        Returns:
            str: Date of estimate.

        """
        return self.date
 
    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number
  
    def set_status(self, status):
        """Set status.

        Args:
            status(str): Status.

        """
        self.status = status
    
    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return self.status
 
    def set_customer_id(self, customer_id): 
        """Set customer id.

        Args:
            customer_id(str): Customer id.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns:
            str: Customer id.

        """
        return self.customer_id
  
    def set_customer_name(self, customer_name):
        """Set customer name.

        Args:
            customer_name(str): Customer name.

        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.

        """
        return self.customer_name

    def set_contact_persons(self, contact_person):
        """Set contact person.

        Args:
            contact_person(list of instance): List of contact person object.

        """ 
        self.contact_persons.extend(contact_person)
    def get_contact_persons(self):
        """Get contact person.

        Returns:
            list: List of contact person object.
 
        """
        return self.contact_persons
   
    def set_currency_id(self, currency_id):
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code.
 
        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.
 
        """
        return self.currency_code
  
    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate
 
    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate
  
    def set_expiry_date(self, expiry_date):
        """Set expiry date.

        Args:
            expiry_date(str): Expiry date.

        """
        self.expiry_date = expiry_date

    def get_expiry_date(self):
        """Get expiry date.

        Returns:
            str: Expiry date.

        """
        return self.expiry_date
  
    def set_discount(self, discount):
        """Set discount.

        Args:
            discount(float): Discount.

        """
        self.discount = discount

    def get_discount(self):
        """Get discount.

        Returns:
            str: Discount.

        """
        return self.discount
  
    def set_is_discount_before_tax(self, is_discount_before_tax):
        """Set whether to discount before tax or not.

        Args:
            is_discount_before_tax(bool): True to discount before tax.

        """
        self.is_discount_before_tax = is_discount_before_tax

    def get_is_discount_before_tax(self):
        """Get whether to discount before tax or not.

        Returns:
            bool: True to discount before tax.

        """
        return self.is_discount_before_tax
  
    def set_discount_type(self, discount_type):
        """Set type of discount.

        Args:
            discount_type(str): Discount type.

        """
        self.discount_type = discount_type

    def get_discount_type(self):
        """Get type of discount.

        Returns:
            str: Discount type.
   
        """
        return self.discount_type 
  
  
    def set_line_items(self, items):
        """Set line items.
 
        Args:
            items(instance): Line items object.

        """
        self.line_items.append(items)

    def get_line_items(self):
        """Get line items.

        Returns:
            list of instance: List of line items object.

        """
        return self.line_items
      
    def set_shipping_charge(self, shipping_charge):
        """Set shipping charge.

        Args:
            shipping_charge(float): Shipping charge.

        """
        self.shipping_charge = shipping_charge

    def get_shipping_charge(self):
        """Get shipping charge.

        Returns:
            float: Shipping charge.

        """
        return self.shipping_charge
  
    def set_adjustment(self, adjustment):
        """Set adjustment.

        Args:
            adjustment(float): Adjustment.

        """
        self.adjustment = adjustment

    def get_adjustment(self):
        """Get adjustment.

        Returns:
            float: Adjustment.

        """
        return self.adjustment
  
    def set_adjustment_description(self, adjustment_description):
        """Set adjustment description.

        Args: 
            adjustment_description(str): Adjustment description.

        """
        self.adjustment_description = adjustment_description 

    def get_adjustment_description(self):
        """Get adjustment description.

        Returns:
            str: Adjustment description.

        """
        return self.adjustment_description

    def set_sub_total(self, sub_total):
        """Set sub total.

        Args:
            sub_total(float): Sub total.

        """
        self.sub_total = sub_total

    def get_sub_total(self):
        """Get sub total.

        Returns:
            float: Sub total.

        """
        return self.sub_total

    def set_total(self, total):
        """Set total.

        Args:
            total(float): Total.
     
        """
        self.total = total

    def get_total(self):
        """Get total.

        Returns:
            float: Total.

        """
        return self.total

    def set_price_precision(self, price_precision):
        """Set price precision.

        Args:
            price_precision(int): Price precision.

        """
        self.price_precision = price_precision

    def get_price_precision(self):
        """Get price precision.

        Returns:
            int: Price precision.

        """
        return self.price_precision
  
    def set_taxes(self, tax):
        """Set taxes.

        Args:
            taxes(instance): Tax object.

        """
        self.taxes.extend(tax)

    def get_taxes(self):
        """Get taxes.

        Returns:
            list of instance: List of tax object.

        """
        return self.taxes

    def set_billing_address(self, address):
        """Set billing address.

        Args:
            address(instance): Address object.

        """
        self.billing_address = address

    def get_billing_address(self):
        """Get billing address.

        Returns:
            instance: Address object.

        """
        return self.billing_address

    def set_shipping_address(self, address):
        """Set shipping address.

        Args:
            address(instance): Address object.

        """
        self.shipping_address = address

    def get_shipping_address(self):
        """Get shipping address.

        Returns:
            instance: Address object.

        """
        return self.shipping_address
   
    def set_notes(self, notes):
        """Set note.

        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes
 
    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """
        self.terms = terms

    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms
  
    def set_custom_fields(self, custom_field):
        """Set custom fields.

        Args:
            custom_field(instance): Custom field.

        """
        self.custom_fields.append(custom_field)

    def get_custom_fields(self):
        """Get custom field.

        Returns:
             list of instance: List of custom field object.

        """
        return self.custom_fields
 
    def set_template_id(self, template_id):
        """Set template id.

        Args:
            template_id(str): Template id.

        """
        self.template_id = template_id
    def get_template_id(self):
        """Get template id.

        Returns: 
            str: Template id.

        """
        return self.template_id
   
    def set_template_name(self, template_name):
        """Set template name.

        Args:
            template_name(str): Template name.

        """
        self.template_name = template_name

    def get_template_name(self):
        """Get template name.

        Returns:
            str: Template name.

        """
        return self.template_name
 
    def set_created_time(self, created_time):
        """Set created time.

        Args:
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created time.

        Returns:
            str: Created time.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.

        Args:
            last_modified_time(str): Last modified time.

        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self):
        """Get last modified time.

        Returns:
            str: LAst modified time.

        """
        return self.last_modified_time
 
    def set_salesperson_id(self, salesperson_id):
        """Set salesperson id.

        Args:
            salesperson_id(str): Salesperson id.

        """
        self.salesperson_id = salesperson_id

    def get_salesperson_id(self):
        """Get salesperson id.

        Returns:
            str: Salesperson id.

        """
        return self.salesperson_id

    def set_salesperson_name(self, salesperson_name):
        """Set salesperson_name.

        Args:
            salesperson_name(str): Salesperson name.

        """
        self.salesperson_name = salesperson_name

    def get_salesperson_name(self):
        """Get salesperson name.

        Returns:
            str: Salesperson name.

        """
        return self.salesperson_name

    def set_accepted_date(self, accepted_date):
        """Set accepted date.

        Args:
            accepted_date(str): Accepted date.

        """
        self.accepted_date = accepted_date

    def get_accepted_date(self):
        """Get accepted date.

        Returns:
            str: Accepted date.

        """
        return self.accepted_date
 
    def set_declined_date(self, declined_date):
        """Set declined date.

        Args: 
            declined_date(str): Declined date.

        """
        self.declined_date = declined_date

    def get_declined_date(self):
        """Get declined date.

        Returns:
            str: Declined date.

        """
        return self.declined_date

    def set_tax_total(self, tax_total):
        """Set tax total.

        Args:
            tax_total(float): Tax total.

        """
        self.tax_total = tax_total

    def get_tax_total(self):
        """Get tax total.

        Returns:
            float: Tax total.

        """
        return self.tax_total

    def to_json(self):
        """This method is used to comnvert estimates object to json object.

        Returns: 
            dict: Dictionary containing json object for estimates.

        """
        data = {}
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.estimate_number != '':
            data['estimate_number'] = self.estimate_number
        if self.contact_persons:
            data['contact_persons'] = self.contact_persons
        if self.template_id != '':
            data['template_id'] = self.template_id
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.date != '':
            data['date'] = self.date
        if self.expiry_date != '':
            data['expiry_date'] = self.expiry_date
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.discount > 0:
            data['discount'] = self.discount
        if self.is_discount_before_tax is not None:
            data['is_discount_before_tax'] = self.is_discount_before_tax
        if self.discount_type != '':
            data['discount_type'] = self.discount_type
        if self.salesperson_name != '':
            data['salesperson_name'] = self.salesperson_name
        if self.custom_fields:
            data['custom_fields'] = []
            for value in self.custom_fields:
                custom_field = value.to_json()
                data['custom_fields'].append(custom_field)
        if self.line_items:
            data['line_items'] = []
            for value in self.line_items:
                line_item = value.to_json()
                data['line_items'].append(line_item)
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        if self.shipping_charge > 0:
            data['shipping_charge'] = self.shipping_charge
        if self.adjustment != '':
            data['adjustment'] = self.adjustment
        if self.adjustment_description != '':
            data['adjustment_description'] = self.adjustment_description
        return data




from urllib.parse import urlencode
from urllib.request import urlretrieve
from httplib2 import Http
from json import dumps,loads
from re import search
from mimetypes import guess_type
from string import digits,ascii_letters
from random import choice
from books.exception.BooksException import BooksException

_BOUNDARY_CHARS = digits + ascii_letters

class ZohoHttpClient:
    """This class is used to create get, post, put and delete connections 
        for the request."""

    def get(self, url, details, query=None):
        """This method is used to make get request for the given url and 
            query string.
        
        Args:
            url(str): Url passed by the user.
            details(dict): Dictionary containing authtoken and organization id.
            query(dict, optional): Additional parameters. Default to None.

        Returns:
            dict: Dictionary containing response content.

        """
        http, headers = get_http_connection(details['authtoken'])
        if 'organization_id' in details and details['organization_id'] != '':
            url = url + '?' + form_query_string({'organization_id': details['organization_id']})
        if query is not None: 
            url += '&' + form_query_string(query)
        print("URL: {}".format(url))
        resp, content = http.request(url, 'GET', headers=headers)
        #print content
        response = get_response(resp['status'], content)
        return response

    def getfile(self, url, details, query=None):
        """This method is used to make get request for the given url and 
            query string.

        Args:
            url(str): Url passed by the user.
            details(dict): Dictionary containing authtoken and organization id.
            query(dict, optional): Additional parameters. Default to None.

        Returns:
            dict: Dictionary containing response content.

        """
        http = Http(timeout=60*1000)
        url = url + '?' + form_query_string(details)
        if query is not None:
            url += '&' + form_query_string(query) 
        headers = {
            'Accept': \
            'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
            'Content-type': 'application/x-www-form-urlencoded',
            'User-Agent': 'ZohoBooks-Python-Wrappers/1.0',
            'Accept-Charset': 'UTF-8'
            }
        resp, content = http.request(url, 'GET', headers=headers)
        if resp['status'] == '200' or resp['status'] == '201':
            attachment = resp['content-disposition']
            regex = search(r'".*"',attachment)
            filename = regex.group().strip('"') if regex is not None else \
            'attachment.' + query['accept'] 
            file_location = "/home/likewise-open/ZOHOCORP/chitra-2327/Downloads/"
            file_name = open(file_location + filename, "w")
            file_name.write(content)
            file_name.close()
            return file_name
        else: 
            raise BooksException(convert(loads(content)))
     
    def post(self, url, details, field, query_string=None, attachment=None):
        """This method is used to make post request for the given url 
               and query string.

        Args:
            url(str): Url passed by the user.
            details(dict): Dictionary containing authtoken and organization id.
            data(dict): Dictionary containing required parameters.
            query(dict, optional): Additional parameters. Default to None.
            attachment(dict, None): Files to be attached. Default to None.

        Returns:
            dict: Dictionary containing response content.
        
        """
        http, headers = get_http_connection(details['authtoken'])
        url = url + '?' + form_query_string({'organization_id': details['organization_id']})
        if query_string is not None:  
            url = url + '&' + form_query_string(query_string)
        print("URL: {}".format(url))
        print("Body: {}".format(field))
        if attachment is None:
            body = urlencode(field)
        else:
            body, headers = encode_multipart(field, attachment)
        resp, content = http.request(url, 'POST', headers=headers,
                                         body=body)
        #print content
        response = get_response(resp['status'], content)
        return response

    def put(self, url, details, field, query=None, attachment=None):
        """This method is used to make put request for the given url and 
            query string.

        Args:
            url(str): Url passed by the user.
            details(dict): Dictionary containing authtoken and organization id. 
            data(dict): Dictionary containing required parameters. 
            query(dict, optional): Additional parameters. Default to None.
            attachment(dict, None): Files to be attached. Default to None.

        Returns:
            tuple: Tuple containing response status(str) and content(dict).
        
        """
        http, headers = get_http_connection(details['authtoken'])
        url = url + '?' + form_query_string(details)
        if query is not None:
            url = url + '&' + form_query_string(query)
        print("URL: {}".format(url))
        print("Body: {}".format(field))
        if attachment is None:
            body = urlencode(field)
        else:
            body, headers = encode_multipart(field, attachment)
        resp, content = http.request(url, 'PUT', headers=headers,
                                     body=body)
        #print content
        response = get_response(resp['status'], content)
        return response


    def delete(self, url, details, param=None):
        """This method is used to make delete request for the given url and 
            query string.

        Args:
            url(str): Url passed by the user.
            details(dict): Dictionary containing authtoken and organization id.
            param(dict): Parameters to be passed in query string.

        Returns:
            tuple: Tuple containing response status(str) and content(dict).
        
        """
        http, headers = get_http_connection(details['authtoken'])
        url = url + '?' + form_query_string(details)
        if param is not None:
           url = url + '&' + form_query_string(param)
        response, content = http.request(url, 'DELETE', headers=headers)
        #print content
        response = get_response(response['status'], content)
        return response
   
def form_query_string(parameter):
    """This method is used to form query string.
 
    Args:
        parameter(dict): Parameters to be converted to query string.
 
    Returns:  
        str: Query string.

    """
    query = ''
    length = len(parameter) 
    for key, value in list(parameter.items()):  
        length = length-1
        query += str(key) + '=' + str(value)
        if length != 0:
            query += '&'
    return query

def encode_multipart(fields, file_list, boundary=None):
    """This method is used to encode multipart data.
   
    Args:
        fields(dict): Parameters in key value pair.
        files(dict): Files to be attached.
        boundary(str, optional): Boundary. Default to None.
    
    Returns:
        tuple: Tuple containing body(list) and headers(dict).
        
    """
    def escape_quote(s):
        return s.replace('"', '\\"')

    if boundary is None:
        boundary = ''.join(choice(_BOUNDARY_CHARS) for i in range(30))

    lines = []
    if fields['JSONString'] != '""':
        for name, value in list(fields.items()):
            lines.extend(('--{0}'.format(boundary),
                          'Content-Disposition: form-data; name="{0}"'\
                          .format(escape_quote(name)),
                          '',
                          str(value),
                          ))
    for files in file_list:
        for name, value in list(files.items()):
            filename = value['filename']
            if 'mimetype' in value:
                mimetype = value['mimetype']
            else:
                mimetype = guess_type(filename)[0] or \
                           'application/octet-stream'
            lines.extend(('--{0}'.format(boundary),\
            'Content-Disposition: form-data; name="{0}";filename="{1}"'\
            .format(escape_quote(name),
                          escape_quote(filename)),
                          'Content-Type: {0}'.format(mimetype),
                          '',
                          value['content'],
                          ))

    lines.extend(('--{0}--'.format(boundary),'',))
    body = '\r\n'.join(lines)
    headers = {
               'Content-Type': 'multipart/form-data; boundary={0}'\
               .format(boundary),
               'Content-Length': str(len(body)),
               }
    return (body, headers)

def convert(input):
    """This method is used to convert unicode objects into strings.
   
    Args:
        input(dict): dictionary of unicode objects

    Returns:
        dict: dictionary of string

    """
    if isinstance(input, dict):
        return {convert(key): convert(value) for key, value in \
        input.items()}
    elif isinstance(input, list):
        return [convert(element) for element in input]
    elif isinstance(input, str):
        return input.encode('utf-8')
    else:
        return input

def get_response(status, content):
    """This method checks the status code and returns respective response 
        message or exception.

    Args:
        status(str): Response status code.
        content(dict): Dictionary containing code and message.

    Returns:
        dict: Response message

    Raises:
        Books Exception: If status is not '200' or '201'.

    """
    resp = loads(content)
    response = resp
    if status != '200' and status != '201':
        raise BooksException(response)
    else:
        return response


def get_http_connection(oauth_token):
    http = Http(timeout=60*1000)
    headers = {
            'Accept': 'application/json',
            'Content-type': 'application/x-www-form-urlencoded',
            'User-Agent': 'ZohoBooks-Python-Wrappers/1.0',
            'Accept-Charset': 'UTF-8',
            'Authorization': 'Zoho-oauthtoken {}'.format(oauth_token)
            }
    return http, headers



#$Id$

from books.model.PageContext import PageContext

class InvoiceList:
    """This class is used to create object for InvoiceList."""
    def __init__(self):
        """Initialize parameters for Invoice list."""
        self.invoice = []
        self.page_context = PageContext()

    def set_invoices(self, invoice):
        """Set Invoices.

        Args:
            invoice(instace): Invoice object.
 
        """
        self.invoice.append(invoice)
 
    def get_invoices(self):
        """Get invoices.

        Returns:
            list of instances: List of invoice object.

        """
        return self.invoice

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context
   



#$Id$

class Transaction: 
    """This class is used to create object for transaction."""
    def __init__(self):
        """Initialize parameters for transactions object."""
        self.transaction_id = ''
        self.debit_or_credit = ''
        self.date = ''
        self.customer_id = ''
        self.payee = ''
        self.reference_number = '' 
        self.transaction_type = ''
        self.amount = ''
        self.status = ''
        self.date_formatted = ''
        self.transaction_type_formatted = ''
        self.amount_formatted = ''
        self.customer_name = ''
        self.from_account_id = ''
        self.from_account_name  =  ''
        self.to_account_id = ''
        self.to_account_name = ''
        self.currency_id = ''
        self.currency_code = ''
        self.payment_mode = ''
        self.exchange_rate = 0.0
        self.reference_number = ''
        self.description = ''
        self.imported_transaction_id = ''
        self.associated_transactions = []
        self.categorized_transaction_id = ''
        self.transaction_date = ''
        self.account_id = ''
        self.entry_number = ''
        self.offset_account_name = ''
        self.reconcile_status = ''
        self.debit_amount = 0.0
        self.credit_amount = 0.0
        self.source = ''
        self.imported_transactions = []
        self.status_formatted = ''

    def set_imported_transaction_id(self, imported_transaction_id):
        """Set imported transaction id.

        Args:
            imported_transaction_id(str): Imported transaction id.

        """
        self.imported_transaction_id = imported_transaction_id

    def get_imported_transaction_id(self):
        """Get imported transaction id.

        Returns:
            str: Imported transaction id.

        """
        return self.imported_transaction_id

    def set_transaction_id(self, transaction_id): 
        """Set transaction id.

        Args: 
            transaction_id(str): Transaction_id.

        """
        self.transaction_id = transaction_id

    def get_transaction_id(self):
        """Get transaction id.

        Returns: 
            str: Transaction id.

        """
        return self.transaction_id

    def set_debit_or_credit(self, debit_or_credit):
        """Set whether debit or credit.

        Args:
            debit_or_credit(str): Debit or credit.

        """
        self.debit_or_credit = debit_or_credit

    def get_debit_or_credit(self):
        """Get whether debit or credit.

        Returns: 
            str: Debit or credit.

        """
        return self.debit_or_credit

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date

    def get_date(self):
        """Get date.

        Returns:
            str: Date.

        """
        return self.date

    def set_customer_id(self, customer_id):
        """Set customer id.

        Args:
            customer_id(self): Customer id.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns:
            str: Customer id.

        """
        return self.customer_id

    def set_payee(self, payee):
        """Set payee.

        Args:
            payee(str): Payee.

        """
        self.payee = payee

    def get_payee(self):
        """Get payee.

        Returns:
            str: Payee.

        """
        return self.payee

    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_transaction_type(self, transaction_type):
        """Set transaction type.

        Args:
            transaction_type(str): Transaction type.

        """
        self.transaction_type = transaction_type

    def get_transaction_type(self):
        """Get transaction type.

        Returns: 
            str: Transaction type.

        """
        return self.transaction_type

    def set_amount(self, amount):
        """Set amount.

        Args:
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self):
        """Get amount.

        Returns:
            float: Amount.

        """
        return self.amount

    def set_status(self, status):
        """Set status.

        Args:
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return self.status

    def set_date_formatted(self, date_formatted): 
        """Set date formatted.

        Args:
            date_formatted(str): Date formatted.

        """
        self.date_formatted = date_formatted

    def get_date_formatted(self):
        """Get date formatted.

        Returns:
            str: Date formatted.
       
        """

    def set_transaction_type_formatted(self, transaction_type_formatted):
        """Set transaction type formatted.

        Args:
            transaction_type_formatted(str): Transaction type formatted.

        """
        self.transaction_type_formatted = transaction_type_formatted

    def get_transaction_type_formatted(self):
        """Get transaction type formatted.

        Returns:
            str: Transaction type formatted.

        """
        return self.transaction_type_formatted

    def set_amount_formatted(self, amount_formatted):
        """Set amount formatted.

        Args:
            amount_formatted(str): Amount formatted.

        """
        self.amount_formatted = amount_formatted

    def get_amount_formatted(self):
        """Get amount formatted.

        Returns: 
            str: Amount formatted.

        """
        return self.amount_formatted

    def set_customer_name(self, customer_name): 
        """Set customer name.

        Args:
            customer_name(str): Customer name.
 
        """
        self.customer_name = customer_name
 
    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.
 
        """
        return self.customer_name

    def set_from_account_id(self, from_account_id):
       """Set from account id.

       Args:
           from_account_id(str): From account id.

       """
       self.from_account_id = from_account_id

    def get_from_account_id(self):
        """Get from account id.

        Returns:
            str: From account id.

        """
        return self.from_account_id

    def set_from_account_name(self, from_account_name): 
        """Set from account name.

        Args:
            from_account_name(str): From account name.

        """
        self.from_account_name = from_account_name

    def get_from_account_name(self):
        """Get from account name.

        Returns:
            str: From account name.

        """
        return self.from_account_name

    def set_to_account_id(self, to_account_id):
        """Set to account id.

        Args:
            to_account_id(str): To account id.

        """
        self.to_account_id = to_account_id    

    def get_to_account_id(self):
        """Get to account id.

        Returns:
            str: To account id.

        """
        return self.to_account_id
   
    def set_to_account_name(self, to_account_name): 
        """Set to account name.

        Args:
            to_account_name(str): To account name.

        """
        self.to_account_name = to_account_name

    def get_to_account_name(self):
        """Get to account name.
 
        Returns:
            str: To account name.

        """
        return self.to_account_name


  
    def set_currency_id(self, currency_id): 
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currecny_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns: 
            str: Currecny id.

        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currecny code.

        Args:
            currency_code(str): Currecny code.

        """
        self.currecy_code = currency_code

    def get_currency_code(self):
        """Get currecny code.

        Returns:
            str: Currecny code.

        """
        return self.currecny_code

    def set_payment_mode(self, payment_mode):
        """Set payment mode.

        Args:
            payment_mode(str): Payment mode.

        """
        self.payment_mode = payment_mode

    def get_payment_mode(self):
        """Get payment mode.

        Returns:
            str: Payment mode.

        """
        return self.payment_mode
  
    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchaneg_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate
  
    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description.

        """
        self.description = description
 
    def get_description(self):
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_currency_id(self, currency_id):
        """Set currency id.

        Args: 
            currency_id(str): Currency id.

        """
        self.currecny_id = currency_id

    def get_currency_id(self): 
        """Get currecny id.
 
        Returns:
            str: Currency id.

        """
        return self.currency_id
   
    def set_currency_code(self, currency_code):
        """Set currency code.

        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code

    def get_currecny_code(self):
        """Get currecny code.

        Returns: 
            str: Currency code.

        """
        return self.currency_code

    def set_payment_mode(self, payment_mode):
        """Set payment mode.

        Args:
            payment_mode(str): Payment mode.

        """
        self.payment_mode = payment_mode

    def get_payment_mode(self):
        """Get payment mode.

        Returns:
            str: Payment mode.

        """
        return self.payment_mode

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate
    
    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate

    def set_associated_transactions(self, transactions):
        """Set associated transacctions.

        Args:
            transactions(instance): Transaction object.

        """
        self.associated_transactions.append(transactions)

    def get_associated_transactions(self):
        """Get associated transactions.

        Returns:
            list of instance: List of transactions object.

        """
        return self.associated_transactions

    def set_categorized_transaction_id(self, categorized_transaction_id):
        """Set categorized transaction id.

        Args:
            categorized_transaction_id(str): Categorized transaction id.

        """
        self.categorized_transaction_id = categorized_transaction_id

    def get_categorized_transaction_id(self):
        """Get categorized transaction id.

        Returns:
            str: Categorized transaction id.

        """
        return self.categorized_transaction_id

    def set_transaction_date(self, transaction_date):
        """Set transaction date.

        Args:
            transaction_date(str): Transaction date.

        """
        self.transaction_date = transaction_date

    def get_transaction_date(self):
        """Get transaction date.

        Returns:
            str: Transaction date.

        """
        return self.transaction_date

    def set_account_id(self, account_id):
        """Set account id.

        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self):
        """Get account id.

        Returns:
            str: Account id.

        """
        return self.account_id

    def set_entry_number(self, entry_number):
        """Set entry number.

        Args:
            entry_number(str): Entry number.

        """
        self.entry_number = entry_number

    def get_entry_number(self):
        """Get entry number.

        Returns:
            str: Entry number.

        """
        return self.entry_number

    def set_offset_account_name(self, offset_account_name):
        """Set offset account name.

        Args: 
            offset_account_name(str): Offset account name.

        """
        self.offset_account_name = offset_account_name

    def get_offset_account_name(self):
        """Get offset_account_name.

        Returns:
            str: Offset account name.

        """
        return self.offset_account_name

    def set_reconcile_status(self, reconcile_status):
        """Set reconcile status.

        Args:
            reconcile_status(str): Reconcile status.

        """
        self.reconcile_status = reconcile_status

    def get_reconcile_status(self):
        """Get reconcile status.

        Returns:
            str:  Reconcile status.

        """
        return self.reconcile_status

    def set_debit_amount(self, debit_amount):
        """Set debit amount.

        Args:
            debit_amount(float): Debit amount.

        """
        self.debit_amount = debit_amount

    def get_debit_amount(self): 
        """Get debit amount.

        Returns:
            float: Debit amount.

        """
        return self.debit_amount

    def set_credit_amount(self, credit_amount):
        """Set ccredit amount.

        Args:
            credit_amount(flaot): Credit amount.

        """
        self.credit_amount = credit_amount

    def get_credit_amount(self):
        """Get credit amount.

        Returns:
            float: Credit amount.

        """
        return self.credit_amount

    def set_source(self, source):
        """Set source.

        Args:
            source(str): Source.

        """
        self.source = source

    def get_source(self):
        """Get source.
 
        Returns:
            str: Source.

        """
        return self.source

    def set_imported_transaction(self, imported_transactions):
        """Set imported transactions.

        Args:
            imported_transaction: Imported transaction.

        """
        self.imported_transactions.append(imported_transactions)

    def get_imported_transaction(self): 
        """Get imported transactions.

        Returns:
            list: List of imported transactions.

        """
        return self.imported_transactions

    def set_status_formatted(self, status_formatted):
        """Set status formatted.

        Args:
            status_formatted(str): Status formatted.

        """
        self.status_formatted = status_formatted

    def get_status_formatted(self):
        """Get status formatted.

        Returns:
            str: Status formatted.

        """
        return self.status_formatted

    def to_json(self):
        """This method is used to convert taransaction object to json format.

        Returns:
            dict: Dictionary containing json object for transaction.

        """
        data = {}
        if self.from_account_id != '':
            data['from_account_id'] = self.from_account_id
        if self.to_account_id != '':
            data['to_account_id'] = self.to_account_id
        if self.transaction_type != '':
            data['transaction_type'] = self.transaction_type
        if self.amount > 0:
            data['amount'] = self.amount
        if self.payment_mode != '':
            data['payment_mode'] = self.payment_mode
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.date != '':
            data['date'] = self.date
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.description != '':
            data['description'] = self.description
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        return data




#$Id$

class CreditNoteRefund:
    """This class is used to create object for creditnotes refund."""
    def __init__(self):
        """Initialize parameters for creditnotes refund object."""
        self.date = ''
        self.refund_mode = ''
        self.reference_number = ''
        self.amount = 0.0
        self.exchange_rate = 0.0
        self.from_account_id = ''
        self.from_account_name = ''
        self.description = ''
        self.creditnote_refund_id = ''
        self.creditnote_id = ''
        self.creditnote_number = ''
        self.customer_name = ''
        self.amount_bcy = ''
        self.amount_fcy = ''

    def set_date(self, date):
        """Set the date at which the credit note is created.
        
        Args:
            date(str): Date.
        
        """
        self.date = date

    def get_date(self):
        """Get date at which the credit note is created.
        
        Returns:
            str: Date.

        """ 
        return self.date
  
    def set_refund_mode(self, refund_mode):
        """Set the mode of refund for the credit note refund amount.
       
        Args:
            refund_mode(str): Refund mode .
 
        """
        self.refund_mode = refund_mode
 
    def get_refund_mode(self):
        """Get the mode of refund for the credit note refund amount.
 
        Returns:
            str: Refund mode.

        """
        return self.refund_mode
   
    def set_reference_number(self, reference_number):
        """Set reference number for the refund recorded.
        
        Args:
            reference_number(str): Reference number.
       
        """
        self.reference_number = reference_number
 
    def get_reference_number(self):
        """Get reference number for the refund recorded.
       
        Returns:
            str: Reference number.
       
        """
        return self.reference_number
  
    def set_amount(self, amount):
        """Set amount refunded from the credit note.
        
        Args:
            amount(float): Amount.
  
        """
        self.amount = amount

    def get_amount(self):
        """Get amount refunded from the credit note.
      
        Returns:
            float: Amount.
   
        """
        return self.amount
 
    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate of the currency.
         
        Args: 
            exchange_rate(float): Exchange rate.
        
        """
        self.exchange_rate = exchange_rate
 
    def get_exchange_rate(self):
        """Get exchange rate of the currency.
        
        Returns:
            float: Exchange rate.
            
        """
        return self.exchange_rate
   
    def set_from_account_id(self, from_account_id):
        """Set the account id from which credit note is refunded.
       
        Args:
            from_account_id(str): The account id from which credit note is 
                refunded.

        """
        self.from_account_id = from_account_id

    def get_from_account_id(self):
        """Get the account id from which credit note is refunded.
 
        Returns:
            str: The account id from which credit note is refunded.
         
        """
        return self.from_account_id
   
    def set_description(self, description):
        """Set description for the refund.
        
        Args:
            description(str): Description for the refund.

        """
        self.description = description
 
    def get_description(self):
        """Get descriptioon for the refund.
   
        Returns:
            str: Description for the refund.

        """
        return self.description

    def set_creditnote_refund_id(self, creditnote_refund_id):
        """Set creditnote refund id.
        
        Args:
            creditnote_refund(str): Id of the creditnote refund.
        
        """
        self.creditnote_refund_id = creditnote_refund_id

    def get_creditnote_refund_id(self):
        """Get creditnote refund id.
        
        Returns:
            str: Id of the creditnote refund.
 
        """
        return self.creditnote_refund_id
  
    def set_creditnote_id(self, creditnote_id):
        """Set credit note id.
          
        Args:
            creditnote_id(str): Id of the creditnote.
 
        """
        self.creditnote_id = creditnote_id
 
    def get_creditnote_id(self):
        """Get Id of the credit note.
         
        Returns:
            str: Id of the creditnote.

        """ 
        return self.creditnote_id
  
    def set_from_account_name(self, from_account_name):
        """Set account name from which credit note is refunded.
       
        Args:
            from_account_name(str): Account name from which credit note is 
                refunded.
       
        """ 
        self.from_account_name = from_account_name

    def get_from_account_name(self):
        """Get account name from which credit note is refunded.
       
        Returns:
            str: Account name from which credit note is refunded.
       
        """ 
        return self.from_account_name

    def set_creditnote_number(self, creditnote_number):
        """Set creditnote number of the creditnote.
  
        Args:
            creditnote_number(str): Creditnote number of creditnote.

        """
        self.creditnote_number = creditnote_number

    def get_creditnote_number(self):
        """Get creditnote number of the creditnote.
         
        Returns:
            str: Creditnote number of creditnote.
     
        """
        return self.creditnote_number

    def set_customer_name(self, customer_name):
        """Set customer name.
       
        Args:
            customer_name(str): Customer name.
     
        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.
        
        Returns:
            str: Customer name.
        
        """
        return self.customer_name

    def set_amount_bcy(self, amount_bcy):
        """Set amount bcy.
      
        Args:
            amount_bcy(str): Amount_bcy.
      
        """
        self.amount_bcy = amount_bcy
 
    def get_amount_bcy(self):
        """Get amount bcy.
      
        Returns:
            str: Amount_bcy.

        """
        return self.amount_bcy
  
    def set_amount_fcy(self, amount_fcy):
        """Set amount fcy.
        
        Args: 
            amount_fcy(str): Amount fcy.

        """
        self.amount_fcy = amount_fcy

    def get_amount_fcy(self):
        """Get amount fcy.
      
        Returns: 
            str: Amount fcy.

        """    
        return self.amount_fcy

    def to_json(self):
        """This method is used to convert creditnote refund object to json object.

        Returns:
            dict: Dictionary containing json object for credit note refund.

        """
        data = {}
        if self.date != '':
            data['date'] = self.date
        if self.refund_mode != '':
            data['refund_mode'] = self.refund_mode
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.amount > 0:
            data['amount'] = self.amount
        if self.exchange_rate > 0 :
            data['exchange_rate'] = self.exchange_rate
        if self.from_account_id != '':
            data['from_account_id'] = self.from_account_id
        if self.description != '':
            data['description'] = self.description
        if self.creditnote_id != '':
            data['creditnote_id'] = self.creditnote_id
        return data

  




#$Id$

class InvoiceSetting:
    """This class is used to create object for invoice settings."""
    def __init__(self):
        """Initialize parameters for Invoice settings."""
        self.auto_generate = None
        self.prefix_string = ''
        self.start_at = 0
        self.next_number = ''
        self.quantity_precision = 0 
        self.discount_type = ''
        self.is_discount_before_tax = None
        self.reference_text = ''
        self.default_template_id = ''
        self.notes = ''
        self.terms = ''
        self.is_shipping_charge_required = None
        self.is_adjustment_required = None
        self.is_open_invoice_editable = None
        self.warn_convert_to_open = None
        self.warn_create_creditnotes = None
        self.attach_expense_receipt_to_invoice = ''
        self.invoice_item_type = ''
        self.is_sales_person_required = None
        self.is_show_invoice_setup = None
        self.discount_enabled = None

    def set_discount_enabled(self, discount_enabled):
        """Set discount enabled.

        Args:
            discount_enabled(str): Discount enabled.

        """
        self.discount_enabled = discount_enabled

    def get_discount_enabled(self):
        """Get discount enabled.

        Returns:
            str: Discount enabled.

        """
        return self.discount_enabled

    def set_default_template_id(self, default_template_id):
        """Set default template id.

        Args:
            default_template_id(str): Default template id.

        """
        self.default_template_id = default_template_id

    def get_default_template_id(self):
        """Get default template id.

        Returns:
            str: Default template id.

        """
        return self.default_template_id

    def set_auto_generate(self, auto_generate):
        """Set auto generate. 

        Args:
            auto_generate(bool): Auto generate.

        """
        self.auto_generate = auto_generate

    def get_auto_generate(self):
        """Get auto generate.

        Returns:
            bool: Auto generate.
       
        """
        return self.auto_generate

    def set_prefix_string(self, prefix_string):
        """Set prefix string.

        Args:
            prefix_string(str): Prefix string.

        """
        self.prefix_string = prefix_string

    def get_prefix_string(self):
        """Get prefix string.

        Returns:
            str: Prefix string.

        """
        return self.prefix_string

    def set_start_at(self, start_at):
        """Set start at.

        Args:
            start_at(int): Start at.

        """
        self.start_at = start_at

    def get_start_at(self):
        """Get start at.

        Returns:
            int: Start at.

        """
        return self.start_at

    def set_next_number(self, next_number):
        """Set next number.

        Args:
            next_number(str): Next number.

        """
        self.next_number = next_number

    def get_next_number(self):
        """Get next number.

        Returns:
            str: Next number.

        """
        return self.next_number

    def set_quantity_precision(self, quantity_precision):
        """Set quantity precision.

        Args:
            quantity_precision(int): Quantity precision.

        """
        self.quantity_precision = quantity_precision

    def get_quantity_precision(self):
        """Get quantity precision.

        Returns:
            int: Quantity precision.

        """
        return self.quantity_precision

    def set_discount_type(self, discount_type):
        """Set discount type.

        Args:
            discount_type(str): Discount type.

        """
        self.discount_type = discount_type

    def get_discount_type(self):
        """Get discount type.

        Returns:
            str: Discount type.

        """
        return self.discount_type

    def set_is_discount_before_tax(self, is_discount_before_tax):
        """Set whether it is discount before tax.

        Args:
            is_discount_before_tax(bool): True to discount before tax.

        """
        self.is_discount_before_tax = is_discount_before_tax

    def get_is_discount_before_tax(self):
        """Get whether to discount before tax.

        Returns:
            bool: True to discount before tax else false.

        """
        return self.is_discount_before_tax

    def set_reference_text(self, reference_text):
        """Set reference text.

        Args:
            reference_text(str): Reference text.

        """
        self.reference_text = reference_text

    def get_reference_text(self):
        """Get reference text.

        Returns:
            str: Reference text.

        """
        return self.reference_text

    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """
        self.terms = terms

    def get_terms(self): 
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def set_is_shipping_charge_required(self, is_shipping_charge_required):
        """Set whether shipping charge is required or not.

        Args:
            is_shipping_charge_required(bool): True if shipping charge is 
                required else false.

        """
        self.is_shipping_charge_required = is_shipping_charge_required

    def get_is_shipping_charge_required(self):
        """Get whether shipping charge is required or not.

        Returns:
            bool: True if shipping charge is required or not.

        """
        return self.is_shipping_charge_required

    def set_is_adjustment_required(self, is_adjustment_required):
        """Set whether adjustment is required.
 
        Args:
            is_adjustment_required(bool): True if adjustment is required else 
                false.
         
        """
        self.is_adjustment_required = is_adjustment_required

    def get_is_adjustment_required(self):
        """Get whether adjustment is required.

        Returns:
            bool: True if adjustment is required.

        """ 
        return self.is_adjustment_required

    def set_is_open_invoice_editable(self, is_open_invoice_editable):
        """Set whether open invoice editable.

        Args:
            is_open_invoice_editable(bool): True if open invoice editable else 
                false.

        """
        self.is_open_invoice_editable = is_open_invoice_editable

    def get_is_open_invoice_editable(self):
        """Get whether open invoice editable.
 
        Returns:
            bool: True if open invoice editable else false.

        """
        return self.is_open_invoice_editable

    def set_warn_convert_to_open(self, warn_convert_to_open):
        """Set whether to enable warning while converting to open.

        Args:
            warn_convert_to_open(bool): True to warn while converting to open 
                else false.

        """
        self.warn_convert_to_open = warn_convert_to_open

    def get_warn_convert_to_open(self):
        """Get whether to enable warning while converting to open.

        Returns:
            bool: True if warning while converting to open is enabled else 
                false.

        """
        return self.warn_convert_to_open

    def set_warn_create_creditnotes(self, warn_create_creditnotes):
        """Set whether to enable warning while creating creditnotes.

        Args: 
            warn_create_creditnotes(bool): True to warn while creating 
                creditnotes else false.

        """
        self.warn_create_creditnotes = warn_create_creditnotes

    def get_warn_create_creditnotes(self):
        """Get whether warning while creating creditnotes is enabled or not.

        Returns:
            bool: True to warn while creating creditnotes else false.

        """
        return self.warn_create_creditnotes

    def set_attach_expense_receipt_to_invoice(self, \
        attach_expense_receipt_to_invoice):
        """Set attach expense receipt to invoice.

        Args:
            attach_expense_receipt_to_invoice(str): Attach expense receipt to 
                invoice.

        """
        self.attach_expense_receipt_to_invoice = \
        attach_expense_receipt_to_invoice
   
    def get_attach_expense_receipt_to_invoice(self):
        """Get attach expense receipt to invoice.

        Returns:
            str: Attach expense receipt to invoice.

        """
        return self.attach_expense_receipt_to_invoice

    def set_is_open_invoice_editable(self, is_open_invoice_editable):
        """Set whether to open invoice editable.

        Args:
            is_open_invoice_editable(bool): True to open invoice editable else 
                false.

        """
        self.is_open_invoice_editable = is_open_invoice_editable

    def get_is_open_invoice_editable(self): 
        """Get whether to open invoice editable.

        Returns:
            bool: True to open invoice editable else false.

        """
        return self.is_open_invoice_editable

    def set_is_sales_person_required(self, is_sales_person_required):
        """Set whether sales person is required or not.

        Args:
            is_sales_person_required(bool): True if sales person is required 
                else false.

        """
        self.is_sales_person_required = is_sales_person_required

    def get_is_sales_person_required(self):
        """Get whether sales person is required or not.

        Returns:
            bool: True if sales person is required else false.

        """
        return self.is_sales_person_required

    def set_is_show_invoice_setup(self, is_show_invoice_setup):
        """Set whether to show invoice setup.

        Args:
            is_show_invoice_setup(bool): True to show invoice setup.

        """
        self.is_show_invoice_setup = is_show_invoice_setup

    def get_is_show_invoice_setup(self):
        """Get whether to show invoice setup.
 
        Returns: 
            bool: True to show invoice setup.
 
        """
        return self.is_show_invoice_setup

    def set_invoice_item_type(self, invoice_item_type):
        """Set invoice item type.

        Args:
            invoice_item_type(str): Invoice item type.

        """
        self.invoice_item_type = invoice_item_type

    def get_invoice_item_type(self):
        """Get invoice item type.

        Returns:
            str: Invoice item type.

        """
        return self.invoice_item_type

    def to_json(self): 
        """This method is used to convert invoice settings in json format.

        Returns:    
            dict: Dictionary containing json object for invoice settings.

        """
        data = {}
        if self.auto_generate is not None:
            data['auto_generate'] = self.auto_generate
        if self.prefix_string != None:
            data['prefix_string'] = self.prefix_string
        if self.start_at > 0:
            data['start_at'] = self.start_at
        if self.next_number != '':
            data['next_number'] = self.next_number
        if self.quantity_precision > 0:
            data['quantity_precision'] = self.quantity_precision
        if self.discount_enabled is not None:
            data['discount_enabled'] = self.discount_enabled
        if self.reference_text != '':
            data['reference_text'] = self.reference_text 
        if self.default_template_id != '':
            data['default_template_id'] = self.default_template_id
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        if self.is_shipping_charge_required is not None:
            data['is_shipping_charge_required'] = \
                self.is_shipping_charge_required
        if self.is_adjustment_required is not None:
            data['is_adjustment_required'] = \
                self.is_adjustment_required
        if self.invoice_item_type != '':
            data['invoice_item_type'] = self.invoice_item_type
        if self.discount_type != '':
            data['discount_type'] = self.discount_type
        if self.warn_convert_to_open is not None: 
            data['warn_convert_to_open'] = self.warn_convert_to_open
        if self.warn_create_creditnotes is not None:
            data['warn_create_creditnotes'] = self.warn_create_creditnotes
        if self.is_open_invoice_editable is not None:
            data['is_open_invoice_editable'] = self.is_open_invoice_editable
        if self.is_sales_person_required is not None:
            data['is_sales_person_required'] = self.is_sales_person_required
        return data

         



#$Id$

class TemplateList:
    """This method is used to create object for Template list."""
    def __init__(self):
        """Initilaize parameters for Template list."""
        self.templates = []

    def set_templates(self, template):
        """Set templates.

        Args:
            template(instance): Template object.

        """ 
        self.templates.append(template)

    def get_templates(self):
        """Get templates.

        Returns:
            list of instance: List of templates object.

        """
        return self.templates



#$Id$#

from books.model.RecurringExpense import RecurringExpense
from books.model.RecurringExpenseList import RecurringExpenseList
from books.model.Expense import Expense
from books.model.ExpenseList import ExpenseList
from books.model.Comment import Comment
from books.model.PageContext import PageContext
from books.model.CommentList import CommentList

class RecurringExpensesParser:
    """This class is used to parse the json for recurring expenses. """
    
    def get_list(self, resp):
        """This method parses the given response and returns recurring 
            expenses list object.

        Args:
            resp(dict): Response containing json object for recurring expenses 
                list.

        Returns:
            instance: Recurring expenses list object.

        """
        recurring_expenses_list = RecurringExpenseList()
        for value in resp['recurring_expenses']:
            recurring_expenses = RecurringExpense()
            recurring_expenses.set_recurring_expense_id(\
            value['recurring_expense_id'])
            recurring_expenses.set_recurrence_name(value['recurrence_name'])
            recurring_expenses.set_recurrence_frequency(\
            value['recurrence_frequency'])
            recurring_expenses.set_repeat_every(value['repeat_every'])
            recurring_expenses.set_last_created_date(\
            value['last_created_date'])
            recurring_expenses.set_next_expense_date(\
            value['next_expense_date'])
            recurring_expenses.set_account_name(value['account_name'])
            recurring_expenses.set_paid_through_account_name(\
            value['paid_through_account_name'])
            recurring_expenses.set_description(value['description'])
            recurring_expenses.set_currency_id(value['currency_id'])
            recurring_expenses.set_currency_code(value['currency_code'])
            recurring_expenses.set_total(value['total'])
            recurring_expenses.set_is_billable(value['is_billable'])
            recurring_expenses.set_customer_name(value['customer_name'])
            recurring_expenses.set_vendor_name(value['vendor_name'])
            recurring_expenses.set_status(value['status'])
            recurring_expenses.set_created_time(value['created_time'])
            recurring_expenses_list.set_recurring_expenses(recurring_expenses)
        page_context = resp['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        recurring_expenses_list.set_page_context(page_context_obj)
        return recurring_expenses_list

    def get_recurring_expense(self, resp):
        """This method parses the given response and returns recurring 
            expenses object.

        Args:
            resp(dict): Response containing json object for recurring expenses.

        Returns:
            instance: Recurring expenses object.

        """
        recurring_expense = resp['recurring_expense']
        recurring_expense_obj = RecurringExpense()
        recurring_expense_obj.set_recurring_expense_id(\
        recurring_expense['recurring_expense_id'])
        recurring_expense_obj.set_recurrence_name(\
        recurring_expense['recurrence_name'])
        recurring_expense_obj.set_start_date(recurring_expense['start_date'])
        recurring_expense_obj.set_end_date(recurring_expense['end_date'])
        recurring_expense_obj.set_recurrence_frequency(\
        recurring_expense['recurrence_frequency'])
        recurring_expense_obj.set_repeat_every(\
        recurring_expense['repeat_every'])
        recurring_expense_obj.set_last_created_date(\
        recurring_expense['last_created_date'])
        recurring_expense_obj.set_next_expense_date(\
        recurring_expense['next_expense_date'])
        recurring_expense_obj.set_account_id(recurring_expense['account_id'])
        recurring_expense_obj.set_account_name(\
        recurring_expense['account_name'])
        recurring_expense_obj.set_paid_through_account_id(\
        recurring_expense['paid_through_account_id'])
        recurring_expense_obj.set_paid_through_account_name(\
        recurring_expense['paid_through_account_name'])
        recurring_expense_obj.set_vendor_id(recurring_expense['vendor_id'])
        recurring_expense_obj.set_vendor_name(recurring_expense['vendor_name'])
        recurring_expense_obj.set_currency_id(recurring_expense['currency_id'])
        recurring_expense_obj.set_currency_code(\
        recurring_expense['currency_code'])
        recurring_expense_obj.set_exchange_rate(\
        recurring_expense['exchange_rate'])
        recurring_expense_obj.set_tax_id(recurring_expense['tax_id'])
        recurring_expense_obj.set_tax_name(recurring_expense['tax_name'])
        recurring_expense_obj.set_tax_percentage(\
        recurring_expense['tax_percentage'])
        recurring_expense_obj.set_tax_amount(recurring_expense['tax_amount'])
        recurring_expense_obj.set_sub_total(recurring_expense['sub_total'])
        recurring_expense_obj.set_total(recurring_expense['total'])
        recurring_expense_obj.set_bcy_total(recurring_expense['bcy_total'])
        recurring_expense_obj.set_amount(recurring_expense['amount'])
        recurring_expense_obj.set_description(recurring_expense['description'])
        recurring_expense_obj.set_is_inclusive_tax(\
        recurring_expense['is_inclusive_tax'])
        recurring_expense_obj.set_is_billable(recurring_expense['is_billable'])
        recurring_expense_obj.set_customer_id(recurring_expense['customer_id'])
        recurring_expense_obj.set_customer_name(\
        recurring_expense['customer_name'])
        recurring_expense_obj.set_status(recurring_expense['status'])
        recurring_expense_obj.set_created_time(\
        recurring_expense['created_time'])      
        recurring_expense_obj.set_last_modified_time(\
        recurring_expense['last_modified_time'])
        recurring_expense_obj.set_project_id(recurring_expense['project_id'])
        recurring_expense_obj.set_project_name(\
        recurring_expense['project_name'])
        return recurring_expense_obj

    def get_message(self, response):
        """This method parses the json object and returns the message string.

        Args:
            response(dict): Response containing json object for message.
 
        Returns:
            str: Success message.

        """
        return response['message']

    def get_expense_history(self, resp):
        """This method parses the json object and returns expense history list.

        Args:
            resp(dict): Response containing json object for expense history.

        Returns:
            instance: Expenses list object.

        """
        expenses_list = ExpenseList()
        for value in resp['expensehistory']:
            expense = Expense()
            expense.set_expense_id(value['expense_id'])
            expense.set_date(value['date'])
            expense.set_account_name(value['account_name'])
            expense.set_vendor_name(value['vendor_name'])
            expense.set_paid_through_account_name(\
            value['paid_through_account_name'])
            expense.set_customer_name(value['customer_name'])
            expense.set_total(value['total'])
            expense.set_status(value['status'])
            expenses_list.set_expenses(expense)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        expenses_list.set_page_context(page_context_obj)
        return expenses_list

    def get_comments(self, resp):
        """This method parses the given response and returns comments list.

        Args:
            resp(dict): Response containing comments list.

        Returns:
            instance: Comments List object.

        """
        comments_list = CommentList()
        for value in resp['comments']:
            comment = Comment()
            comment.set_comment_id(value['comment_id'])
            comment.set_recurring_expense_id(value['recurring_expense_id'])
            comment.set_description(value['description'])
            comment.set_commented_by_id(value['commented_by_id'])
            comment.set_commented_by(value['commented_by'])
            comment.set_date(value['date'])
            comment.set_date_description(value['date_description'])
            comment.set_time(value['time'])
            comment.set_operation_type(value['operation_type'])
            comment.set_transaction_id(value['transaction_id'])
            comment.set_transaction_type(value['transaction_type'])
            comments_list.set_comments(comment)
        return comments_list



#$Id$

class User:
    """This class is used to create object for Users."""
    def __init__(self):
        """Initialize parameters for Users object."""
        self.user_id = ''
        self.is_current_user = None
        self.user_name = ''
        self.email = ''
        self.user_role = ''
        self.status = ''
        self.rate = 0.0
        self.budget_hours = 0
        self.total_hours = ''
        self.billed_hours = ''
        self.un_billed_hours = ''
        self.project_id = ''
        self.created_time = ''
        self.email_ids = []
        self.role_id = ''
        self.name = ''
        self.photo_url = ''
    
    def set_photo_url(self, photo_url):
        """Set photo url.

        Args:
            photo_url(str): Photo url.

        """
        self.photo_url = photo_url

    def get_photo_url(self):
        """Get photo url.

        Returns:
            str: Photo url.

        """
        return self.photo_url

    def set_name(self, name):
        """Set name.

        Args:
            name(str): Name.
 
        """
        self.name = name

    def get_name(self):
        """Get name.
 
        Returns:
            str: Name.

        """
        return self.name

    def set_project_id(self, project_id):
        """Set project id.

        Args:
            project_id(str): Project id.

        """
        self.project_id = project_id

    def get_project_id(self):
        """Get project id.

        Returns:
            str: Project id.

        """
        return self.project_id 

    def set_user_id(self, user_id):
        """Set user id.

        Args:
            user_id(str): User id.

        """
        self.user_id = user_id

    def get_user_id(self):
        """Get user id.

        Returns:
            str: User id.

        """
        return self.user_id

    def set_is_current_user(self, is_current_user):
        """Set whether it is current user or not.

        Args:
            is_current_user(bool): True if it is current user else False.

        """
        self.is_current_user = is_current_user

    def get_is_current_user(self):
        """Get whether it is current user.

        Returns:
            bool: True if it is a current user else False.

        """
        return self.is_current_user

    def set_user_name(self, user_name): 
        """Set user name.

        Args:
            user_name(str): User name.

        """
        self.user_name = user_name

    def get_user_name(self):
        """Get user name.

        Returns:
            str: User name

        """
        return self.user_name

    def set_email(self, email):
        """Set email.

        Args:
            email(str): Email.

        """
        self.email = email

    def get_email(self):
        """Get email.

        Returns:
            str: Email.

        """
        return self.email

    def set_user_role(self, user_role):
        """Set user role.

        Args:
            user_role(str): User role.

        """
        self.user_role = user_role

    def get_user_role(self):
        """Get user role.

        Returns:
            str: User role.

        """
        return self.user_role

    def set_status(self, status):
        """Set status.

        Args:
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """ 
        return self.status

    def set_rate(self, rate):
        """Set rate.

        Args:
            rate(float): Rate.

        """
        self.rate = rate

    def get_rate(self):
        """Get rate.

        Returns:
            float: Rate.

        """
        return self.rate

    def set_budget_hours(self, budget_hours): 
        """Set budget hours.

        Args:
            budget_hours(int): Budget hours.

        """
        self.budget_hours = budget_hours

    def get_budget_hours(self):
        """Get budget hours.

        Returns:
            int: Budget hours.

        """
        return self.budget_hours

    def set_total_hours(self, total_hours):
        """Set total hours.

        Args:
            total_hours(str): Total hours.
   
        """
        self.total_hours = total_hours

    def get_total_hours(self):
        """Get total hours.

        Returns:
            str: Total hours.

        """
        return self.total_hours

    def set_billed_hours(self, billed_hours): 
        """Set billed hours.
   
        Args:
            billed_hours(str): Billed hours.

        """
        self.billed_hours = billed_hours

    def get_billed_hours(self):
        """Get billed hours.

        Returns:
            str: Billed hours.

        """
        return self.billed_hours

    def set_status(self, status): 
        """Set status.

        Args: 
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return self.status

    def set_rate(self, rate):
        """Set rate.
        
        Args:
            rate(float): Rate

        """
        self.rate = rate

    def get_rate(self):
        """Get rate.

        Returns:
            float: Rate.

        """
        return self.rate

    def budget_hours(self, budget_hours):
        """Set budget hours.

        Args:
            budget_hours(str): Budget hours.

        """
        self.budget_hours = budget_hours

    def get_budget_hours(self):
        """Get budget hours.

        Returns:
            str: Budget hours.

        """
        return self.budget_hours

    def set_total_hours(self, total_hours):
        """Set total hours.

        Args:
            total_hours(str): Total hours.

        """
        self.total_hours = total_hours

    def get_total_hours(self):
        """Get total hours.

        Returns:
            str: Total hours.

        """
        return self.total_hours

    def set_billed_hours(self, billed_hours):
        """Set billed hours.

        Args: 
            billed_hours(str): Billed hours.

        """
        self.billed_hours = billed_hours

    def get_billed_hours(self):
        """Get billed hours.

        Returns:
            str: Billed hours.

        """ 
        return self.billed_hours

    def set_un_billed_hours(self, un_billed_hours):
        """Set unbilled hours.

        Args:
            un_billed_hours(str): Un billed hours.

        """
        self.un_billed_hours = un_billed_hours

    def get_un_billed_hours(self):
        """Get unbilled hours.

        Returns:
            str: Unbilled hours.

        """ 
        return self.un_billed_hours

    def set_created_time(self, created_time):
        """Set created time.

        Args:
            created_time(str): Created time.
      
        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created time.
 
        Returns:
            str: Created time.

        """
        return self.created_time

    def set_email_ids(self, email_ids):
        """Set email ids.

        Args:
            email_ids(instance): Email ids object.

        """
        self.email_ids = email_ids

    def get_email_ids(self):
        """Get email ids.
 
        Returns:
            list of instance: list of Email ids object.

        """ 
        return self.email_ids

    def set_role_id(self, role_id):
        """Set role id.

        Args:
            role_id(str): Role id.

        """
        self.role_id = role_id

    def get_role_id(self):
        """Get role id.

        Returns:
            str: Role id.

        """
        return self.role_id

    def to_json(self):
        """This method is used to convert user object to json format.

        Returns:
            dict: Dictionary containing json object for user.

        """
        data = {}
        if self.user_id != '':
            data['user_id'] = self.user_id
        if self.rate > 0:
            data['rate'] = self.rate
        if self.budget_hours > 0:
            data['budget_hours'] = self.budget_hours
        if self.user_name != '':
            data['user_name'] = self.user_name
        if self.email != '':
            data['email'] = self.email
        if self.user_role != '':
            data['user_role'] = self.user_role
        if self.name != '':
            data['name'] = self.name
        return data
        



#$Id$

class PaymentGateway:
    """This class is used to create object for payment gateway object."""
    def __init__(self):
        """Initialize parameters for Payment gateway object."""
        self.gateway_name = ''
        self.gateway_name_formatted = ''
        self.identifier = ''
        self.additional_field1 = ''
        self.additional_field2 = ''
        self.additional_field3 = ''
        self.deposit_to_account_id = ''
        self.deposit_to_account_name = ''
        self.configured = None

    def set_gateway_name(self, gateway_name):
        """Set gateway name associated with recurring profile.

        Args:
            gateway_name(str): Gateway name associated with recurring profile.
                Allowed values are paypal,  authorize_net,  payflow_pro,  
                 stripe, 2checkout and braintree.

        """
        self.gateway_name = gateway_name

    def get_gateway_name(self):
        """Get gateway name.

        Returns:
            str: Gateway name associated with recurring profile.
  
        """
        return self.gateway_name 
  
    def set_gateway_name_formatted(self, gateway_name_formatted):
        """Set gateway name formatted.

        Args:
            gateway_name_formatted(str): Gateway name formatted.
 
        """
        self.gateway_name_formatted = gateway_name_formatted
 
    def get_gateway_name_formatted(self):
        """Get gateway name formatted.

        Returns:
            str: Gateway name formatted.

        """
        return self.gateway_name_formatted
  
    def set_identifier(self, identifier):
        """Set identifier.

        Args: 
            identifier(str): Identifier.

        """
        self.identifier = identifier

    def get_identifier(self):
        """Get identifier.

        Returns:
            str: Identifier.

        """ 
        return self.identifier
  
    def set_additional_field1(self, additional_field1):
        """Set additional field1.  
        
        Args: 
            additional_field1(str): Paypal payment method.Allowed values are 
                standard and adaptive.
 
        """
        self.additional_field1 = additional_field1

    def get_additional_field1(self):
        """Get additional field1.

        Returns:
            str: Paypal payment method.

        """
        return self.additional_field1

    def set_additional_field2(self, additional_field2):
        """Set additional field2.  
        
        Args: 
            additional_field2(str): Paypal payment method.Allowed values are 
                standard and adaptive.
 
        """
        self.additional_field2 = additional_field2
  
    def get_additional_field2(self):
        """Get additional field2.

        Returns:
            str: Paypal payment method.

        """
        return self.additional_field2

    def set_additional_field3(self, additional_field3):
        """Set additional field3.  
        
        Args: 
            additional_field3(str): Paypal payment method.Allowed values are 
                standard and adaptive.
 
        """ 
        self.additional_field3 = additional_field3
  
    def get_additional_field3(self):
        """Get additional fiield3.

        Returns:
            str: Paypal payment method.

        """
        return self.additional_field3

    def set_deposit_to_account_id(self, deposit_to_account_id):
        """Set deposit to account id.

        Args:
            deposit_to_account_id(str): Deposit to account id.

        """
        self.deposit_to_account_id = deposit_to_account_id

    def get_deposit_to_account_id(self):
        """Get deposit to account id.

        Returns:
            str: Deposit to account id.

        """
        return self.deposit_to_account_id

    def set_deposit_to_account_name(self, deposit_to_account_name):
        """Set deposit to account name.
      
        Args: 
            deposit_to_account_name(str): Deposit to account name.

        """
        self.deposit_to_account_name = deposit_to_account_name

    def get_deposit_to_account_name(self):
        """Get deposit to account name.

        Returns:
            str: Deposit to account name.

        """
        return self.deposit_to_account_name

    def set_configured(self, configured):
        """Set Whether to configure or not.

        Args:
            configured(bool): Configured.

        """
        self.configured = configured

    def get_configured(self):
        """Get configured.
 
        Returns: 
            bool: Configured.

        """
        return self.configured

    def to_json(self):
        """This method is used to convert payment gateway object to json object.

        Returns:
            dict: Dictionary containing json object for payment gatewqay.

        """
        data = {}
        if self.gateway_name != '':
            data['gateway_name'] = self.gateway_name 
        if self.additional_field1 != '':
            data['additional_field1'] = self.additional_field1
        return data



#$Id$

from books.model.PlaceHolder import PlaceHolder

class ManualReminder:
    """This class is used to create object for manual reminders."""
    def __init__(self):
        """Initilaize parameters for manual reminders."""
        self.manualreminder_id = ''
        self.type = ''
        self.subject = ''
        self.body = ''
        self.cc_me = None
        self.placeholder = PlaceHolder()

    def set_manualreminder_id(self, manualreminder_id):
        """Set manual reminder id.

        Args:
            manualreminder_id(str): Manual reminder id.

        """
        self.manualreminder_id = manualreminder_id

    def get_manualreminder_id(self):
        """Get manual reminder id.

        Returns:
            str: Manual reminder id.

        """
        return self.manualreminder_id

    def set_type(self, type):
        """Set type.

        Args:
            type(str): Type.

        """
        self.type = type

    def get_type(self):
        """Get type.

        Returns:
            str: Type.

        """
        return self.type

    def set_subject(self, subject):
        """Set subject.

        Args:
            subject(str): Subject.

        """
        self.subject = subject

    def get_subject(self):
        """Get subject.

        Returns:
            str: Subject.

        """
        return self.subject

    def set_body(self, body):
        """Set body.

        Args:
            body(str): Body.

        """
        self.body = body

    def get_body(self):
        """Get body.

        Returns:
            str: Body.

        """
        return self.body

    def set_cc_me(self, cc_me):
        """Set cc me.

        Args:
            cc_me(bool): True to cc me else false.

        """
        self.cc_me = cc_me

    def get_cc_me(self):
        """Get cc me.

        Returns: 
            bool: True to cc me else false.

        """
        return self.cc_me

    def set_placeholders(self, placeholders):
        """Set place holders.

        Args:
            place_holders: Palce holders object.

        """
        self.placeholders = placeholders

    def get_placeholders(self):
        """Get place holders.

        Returns:
            instance: Place holders.

        """
        return self.placeholders

    def to_json(self): 
        """"This method is used to convert manual reminder object to json format.

        Returns:
            dict: Dictionary containing json object for manual reminders.

        """
        data = {}
        if self.subject != '':
            data['subject'] = self.subject
        if self.body != '':
            data['body'] = self.body
        if self.cc_me is not None: 
            data['cc_me'] = self.cc_me
        return data
 



#$Id$

from books.model.PlaceHolder import PlaceHolder

class Autoreminder: 
    """This class is used to create object for auto reminders."""
    def __init__(self):
        """Initialize parameters for auto reminders object."""
        self.payment_reminder_id = ''
        self.is_enabled = None
        self.notification_type = ''
        self.address_type = ''
        self.number_of_days = 0
        self.subject = ''
        self.body = ''
        self.autoreminder_id = ''
        self.order = 0
        self.name = ''
        self.cc_addresses = ''
        self.placeholder = PlaceHolder()
    
    def set_cc_addresses(self, cc_addresses):
        """Set cc addresses.

        Args:
            cc_addresses(str): Cc addresses.

        """
        self.cc_addresses = cc_addresses

    def get_cc_addresses(self):
        """Get cc addresses.

        Returns:
            str: Cc addresses.

        """
        return self.cc_addresses

    def set_name(self, name):
        """Set name.

        Args:
            name(str): Name.

        """
        self.name = name

    def get_name(self):
        """Get name.

        Returns:
            str: Name.

        """
        return self.name

    def set_payment_reminder_id(self, payment_reminder_id):
        """Set payment reminder id.

        Args:
            payment_reminder_id(str): Payment reminder id.

        """ 
        self.payment_reminder_id = payment_reminder_id

    def get_payment_reminder_id(self):
        """Get payment reminder id.

        Returns:
            str: Payment reminder id.

        """
        return self.payment_reminder_id

    def set_is_enabled(self, is_enabled):
        """Set whether it is enabled or not.

        Args:
            is_enabled(bool): True to enable else False.

        """
        self.is_enabled = is_enabled

    def get_is_enabled(self):
        """Get is enabled.

        Returns:
            bool: True if enabled else false.

        """
        return self.is_enabled

    def set_notification_type(self, notification_type):
        """Set notification type.

        Args:
            notification_type(str): Notification type.

        """
        self.notification_type = notification_type

    def get_notification_type(self):
        """Get notification type.
 
        Returns: 
            str: Notification type.

        """
        return self.notification_type

    def set_address_type(self, address_type):
        """Set address type.

        Args:
            address_type(str): Address type.

        """
        self.address_type = address_type

    def get_address_type(self):
        """Get address type.

        Returns:
            str: Address type.

        """
        return self.address_type

    def set_number_of_days(self, number_of_days):
        """Set number of days.

        Args:
            number_of_days(int): Number of days.

        """
        self.number_of_days = number_of_days

    def get_number_of_days(self):
        """Get number of days.

        Returns:
            int: Number of days.

        """
        return self.number_of_days

    def set_subject(self, subject):
        """Set subject.

        Args:
            subject(str): Subject.

        """
        self.subject = subject

    def get_subject(self):
         """Get subject.

         Returns:
             str: Subject.

         """
         return self.subject

    def set_body(self, body):
        """Set body.

        Args:
            body(str): Body.

        """
        self.body = body

    def get_body(self):
        """Get body.

        Returns:
            str: Body.

        """
        return self.body

    def set_autoreminder_id(self, autoreminder_id):
        """Set autoreminder id.

        Args:
            autoreminder_id(str): Auto reminder id.

        """
        self.autoreminder_id = autoreminder_id

    def get_autoreminder_id(self):
        """Get autoreminder id.

        Returns:
            str: Auto reminder id.

        """ 
        return self.autoreminder_id

    def set_order(self, order):
        """Set order.

        Args:
            order(int): Order.

        """
        self.order = order

    def get_order(self):
        """Get order.

        Returns:
            int: Order.

        """
        return self.order

    def set_placeholders(self, placeholders):
        """Set place holders.

        Args:
            place_holders: Palce holders object.

        """
        self.placeholders = placeholders

    def get_placeholders(self):
        """Get place holders.

        Returns:
            instance: Place holders.

        """
        return self.placeholders
 
    def to_json(self):
        """This method is used to convert auto reminder object to json object.

        Returns:
            dict: Dictionary cocntaining json object for autoreminder.

        """
        data = {}
        if self.is_enabled != '':
            data['is_enabled'] = self.is_enabled
        if self.notification_type != '':
            data['type'] = self.notification_type
        if self.address_type != '':
            data['address_type'] = self.address_type
        if self.subject != '':
            data['subject'] = self.subject
        if self.body != '':
            data['body'] = self.body
        return data



#$Id$

class Criteria: 
    """This class is used to create object for criteria."""
    def __init__(self):
        """Initialize parameters for criteria object."""
        self.criteria_id = ''
        self.field = ''
        self.comparator = ''
        self.value = 0.0

    def set_criteria_id(self, criteria_id):
        """Set criteria id.

        Args:
            criteria_id(str): Criteria id.

        """
        self.criteria_id = criteria_id

    def get_criteria_id(self):
        """Get criteria id.

        Returns:
            str: Criteria id.

        """
        return self.criteria_id

    def set_field(self, field):
        """Set field.

        Args:
            field(str): Field.
 
        """
        self.field = field

    def get_field(self): 
        """Get field.

        Returns:
            str: Field.

        """
        return self.field

    def set_comparator(self, comparator):
        """Set comparator.

        Args:
            comparator(str): Comparator.

        """
        self.comparator = comparator

    def get_comparator(self):
        """Get comparator.
 
        Returns:
            str: Comparator.

        """
        return self.comparator

    def set_value(self, value):
        """Set value.

        Args:
            value(float): Value.

        """
        self.value = value

    def get_value(self):
        """Get value.

        Returns:
            float: Value.

        """
        return self.value

    def to_json(self):
        """This method is used to convert criteria object to json object.

        Returns:
            dict: Dictionary containing json object for criteria.

        """
        data = {}
        if self.field != '':
            data['field'] = self.field
        if self.comparator != '':
            data['comparator'] = self.comparator
        if self.value > 0:
            data['value'] = self.value
        return data
          
       



#$Id$

class Term:
    """This class is used to create object for terms."""
    def __init__(self):
        """Initialize parameters for terms."""
        self.invoice_terms = ''
        self.estimate_terms = ''
        self.creditnotes_terms = ''

    def set_invoice_terms(self, invoice_terms):
        """Set invoice terms.

        Args:
            invoice_terms(str): Invoice terms.

        """
        self.invoice_terms = invoice_terms

    def get_invoice_terms(self): 
        """Get invoice terms.

        Returns: 
            str: Invoice terms.

        """ 
        return self.invoice_terms

    def set_estimate_terms(self, estimate_terms):
        """Set estimate terms.

        Args:
            estimate_terms(str): Estimate terms.

        """
        self.estimate_terms = estimate_terms

    def set_creditnote_terms(self, creditnote_terms):
        """Set creditnote terms.

        Args:
            creditnote_terms(str): Creditnote terms.

        """
        self.creditnote_terms = creditnote_terms

    def get_creditnote_terms(self):
        """Get creditnote terms.

        Returns:
            str: Creditnote terms.

        """
        return self.creditnote_terms




#$Id$#

from books.model.Invoice import Invoice
from books.model.PageContext import PageContext
from books.model.InvoiceList import InvoiceList
from books.model.LineItem import LineItem
from books.model.Tax import Tax
from books.model.CustomField import CustomField
from books.model.Address import Address
from books.model.PaymentGateway import PaymentGateway
from books.model.Email import Email
from books.model.FromEmail import FromEmail
from books.model.ToContact import ToContact
from books.model.EmailTemplate import EmailTemplate
from books.model.Template import Template
from books.model.InvoicePayment import InvoicePayment
from books.model.InvoiceCredited import InvoiceCredited
from books.model.InvoiceCreditedList import InvoiceCreditedList
from books.model.Comment import Comment
from books.model.PaymentAndCredit import PaymentAndCredit
from books.model.CommentList import CommentList
from books.model.PaymentList import PaymentList
from books.model.TemplateList import TemplateList

class InvoicesParser:
    """This class is used to parse the json for Invoices."""
 
    def get_list(self,response):
        """This method parses the given resonse and creates invoice list 
            object.
       
        Args:
            response(dict): Response containing json object for invoice list.

        Returns:
            instance: Invoice list object.

        """
        invoice = response['invoices']
        invoice_list = InvoiceList()
        for value in invoice: 
            invoice_object = Invoice()
            invoice_object.set_invoice_id(value['invoice_id'])
            invoice_object.set_customer_name(value['customer_name'])
            invoice_object.set_customer_id(value['customer_id'])
            invoice_object.set_status(value['status'])
            invoice_object.set_invoice_number(value['invoice_number'])
            invoice_object.set_reference_number(value['reference_number'])
            invoice_object.set_date(value['date'])
            invoice_object.set_due_date(value['due_date'])
            invoice_object.set_due_days(value['due_days'])
            invoice_object.set_currency_id(value['currency_id'])
            invoice_object.set_currency_code(value['currency_code'])
            invoice_object.set_total(value['total'])
            invoice_object.set_balance(value['balance'])
            invoice_object.set_created_time(value['created_time'])
            invoice_object.set_is_emailed(value['is_emailed'])
            invoice_object.set_reminders_sent(value['reminders_sent'])
            invoice_object.set_payment_expected_date(value[\
            'payment_expected_date'])
            invoice_object.set_last_payment_date(value['last_payment_date'])
            invoice_list.set_invoices(invoice_object)
        page_context_object = PageContext()
        page_context = response['page_context']
        page_context_object.set_page(page_context['page'])
        page_context_object.set_per_page(page_context['per_page'])
        page_context_object.set_has_more_page(page_context['has_more_page'])
        page_context_object.set_report_name(page_context['report_name'])
        page_context_object.set_applied_filter(page_context['applied_filter'])
        page_context_object.set_sort_column(page_context['sort_column'])
        page_context_object.set_sort_order(page_context['sort_order'])
        invoice_list.set_page_context(page_context_object)
  
        return invoice_list
     

    def get(self,response):
        """This method parses the given response and returns Invoice object.

        Args:
            response(dict): Request containing json obect for invoices.

        Returns:
            instance: Invoice object.

        """
        invoice = response['invoice']
        invoice_object = Invoice()
        invoice_object.set_invoice_id(invoice['invoice_id'])
        invoice_object.set_invoice_number(invoice['invoice_number'])
        invoice_object.set_date(invoice['date'])
        invoice_object.set_status(invoice['status'])
        invoice_object.set_payment_terms(invoice['payment_terms'])
        invoice_object.set_payment_terms_label(invoice['payment_terms_label'])
        invoice_object.set_due_date(invoice['due_date'])
        invoice_object.set_payment_expected_date(invoice[\
        'payment_expected_date'])
        invoice_object.set_last_payment_date(invoice['last_payment_date'])
        invoice_object.set_reference_number(invoice['reference_number'])
        invoice_object.set_customer_id(invoice['customer_id'])
        invoice_object.set_customer_name(invoice['customer_name'])
        invoice_object.set_contact_persons(invoice['contact_persons_details'])
        invoice_object.set_currency_id(invoice['currency_id'])
        invoice_object.set_currency_code(invoice['currency_code'])
        invoice_object.set_exchange_rate(invoice['exchange_rate'])
        invoice_object.set_discount(invoice['discount'])
        invoice_object.gst_no = invoice['gst_no']
        invoice_object.set_is_discount_before_tax(invoice[\
        'is_discount_before_tax'])
        invoice_object.set_discount_type(invoice['discount_type'])
        invoice_object.set_recurring_invoice_id(invoice[\
        'recurring_invoice_id'])
        line_items = invoice['line_items']
        for value in line_items:
            line_item = LineItem()
            line_item.set_line_item_id(value['line_item_id'])
            line_item.set_item_id(value['item_id'])
            line_item.set_project_id(value['project_id'])
            line_item.set_time_entry_ids(value['time_entry_ids'])
            line_item.set_expense_id(value['expense_id'])
            line_item.set_expense_receipt_name(value['expense_receipt_name'])
            line_item.set_name(value['name'])
            line_item.set_description(value['description'])
            line_item.set_item_order(value['item_order'])
            line_item.set_bcy_rate(value['bcy_rate'])
            line_item.set_rate(value['rate'])
            line_item.set_quantity(value['quantity'])
            line_item.set_unit(value['unit'])
            line_item.set_discount(value['discount'])
            line_item.set_tax_id(value['tax_id'])
            line_item.set_tax_name(value['tax_name'])
            line_item.set_tax_type(value['tax_type'])
            line_item.set_tax_percentage(value['tax_percentage'])
            line_item.set_hsn_or_sac(value['hsn_or_sac'])
            line_item.set_item_total(value['item_total'])
            invoice_object.set_line_items(line_item)
        invoice_object.set_shipping_charge(invoice['shipping_charge'])
        invoice_object.set_adjustment(invoice['adjustment'])
        invoice_object.set_adjustment_description(invoice[\
        'adjustment_description'])
        invoice_object.set_sub_total(invoice['sub_total'])
        invoice_object.set_tax_total(invoice['tax_total'])
        invoice_object.set_total(invoice['total'])
        taxes = invoice['taxes']
        for value in taxes:
            tax = Tax()
            tax.set_tax_name(value['tax_name'])
            tax.set_tax_amount(value['tax_amount'])
            invoice_object.set_taxes(tax)
        invoice_object.set_payment_reminder_enabled(invoice[\
        'payment_reminder_enabled'])
        invoice_object.set_payment_made(invoice['payment_made'])
        invoice_object.set_credits_applied(invoice['credits_applied'])
        invoice_object.set_tax_amount_withheld(invoice['tax_amount_withheld'])
        invoice_object.set_balance(invoice['balance'])
        invoice_object.set_write_off_amount(invoice['write_off_amount'])
        invoice_object.set_allow_partial_payments(invoice[\
        'allow_partial_payments'])
        invoice_object.set_price_precision(invoice['price_precision'])
        payment_gateways = invoice['payment_options']['payment_gateways']
        for value in payment_gateways:
            payment_gateway = PaymentGateway()
            payment_gateway.set_configured(value['configured'])
            payment_gateway.set_additional_field1(value['additional_field1'])
            payment_gateway.set_gateway_name(value['gateway_name'])
            invoice_object.set_payment_options(payment_gateway)
        invoice_object.set_is_emailed(invoice['is_emailed'])
        invoice_object.set_reminders_sent(invoice['reminders_sent'])
        invoice_object.set_last_reminder_sent_date(invoice[\
        'last_reminder_sent_date'])
        billing_address = invoice['billing_address']
        billing_address_object = Address()
        billing_address_object.set_address(billing_address['address'])
        billing_address_object.set_city(billing_address['city'])
        billing_address_object.set_state(billing_address['state'])
        billing_address_object.set_zip(billing_address['zip'])
        billing_address_object.set_country(billing_address['country'])
        billing_address_object.set_fax(billing_address['fax'])
        invoice_object.set_billing_address(billing_address_object)
     
        shipping_address = invoice['shipping_address']
        shipping_address_object = Address()
        shipping_address_object.set_address(shipping_address['address'])
        shipping_address_object.set_city(shipping_address['city'])
        shipping_address_object.set_state(shipping_address['state'])
        shipping_address_object.set_zip(shipping_address['zip'])
        shipping_address_object.set_country(shipping_address['country'])
        shipping_address_object.set_fax(shipping_address['fax'])
        invoice_object.set_shipping_address(shipping_address_object)
        invoice_object.set_notes(invoice['notes'])
        invoice_object.set_terms(invoice['terms'])
        custom_fields = invoice['custom_fields']
        for value in custom_fields:
            custom_field = CustomField() 
            custom_field.set_index(value['index'])
            custom_field.set_show_on_pdf(value['show_on_pdf'])
            custom_field.set_value(value['value'])
            custom_field.set_label(value['label'])
            invoice_object.set_custom_fields(custom_field)
        invoice_object.set_template_id(invoice['template_id'])
        invoice_object.set_template_name(invoice['template_name'])
        invoice_object.set_created_time(invoice['created_time'])
        invoice_object.set_last_modified_time(invoice['last_modified_time'])
        invoice_object.set_attachment_name(invoice['attachment_name'])
        invoice_object.set_can_send_in_mail(invoice['can_send_in_mail'])
        invoice_object.set_salesperson_id(invoice['salesperson_id'])
        invoice_object.set_salesperson_name(invoice['salesperson_name'])
  
        return invoice_object

    def get_message(self,response):
        """This message parses the given response and returns string.

        Args:
            response(dict): Request containing json object.

        Returns:
            str: Success message.
      
        """
        return response['message']
     
    def get_content(self,response): 
        """This message parses the given response and returns email object.
 
        Args:
            response(dict): Request containing json object for email content.

        Returns:
            instance: Email object.

        """
        data = response['data']
        email = Email()
        email.set_gateways_configured(data['gateways_configured'])
        email.set_deprecated_placeholders_used(data[\
        'deprecated_placeholders_used'])
        email.set_body(data['body'])
        email.set_error_list(data['error_list'])
        email.set_subject(data['subject'])
        for value in data['emailtemplates']:
            email_template = EmailTemplate()
            email_template.set_selected(value['selected'])
            email_template.set_name(value['name'])
            email_template.set_email_template_id(value['email_template_id'])
            email.set_email_templates(email_template)
        for value in data['to_contacts']:
            to_contact = ToContact()
            to_contact.set_first_name(value['first_name'])
            to_contact.set_selected(value['selected'])
            to_contact.set_phone(value['phone'])
            to_contact.set_email(value['email'])
            to_contact.set_contact_person_id(value['contact_person_id'])
            to_contact.set_last_name(value['last_name'])
            to_contact.set_salutation(value['salutation'])
            to_contact.set_mobile(value['mobile'])   
            email.set_to_contacts(to_contact)
        email.set_attachment_name(data['attachment_name'])
        email.set_file_name(data['file_name'])

        for value in data['from_emails']:
            from_email = FromEmail()
            from_email.set_user_name(value['user_name'])
            from_email.set_selected(value['selected'])
            from_email.set_email(value['email'])
            email.set_from_emails(from_email)
        email.set_customer_id(data['customer_id'])
        return email

    def payment_reminder_mail_content(self,response):
        """This method parses the given response and returns the email object.

        Args:
            response(dict): Response containing json object for email content.

        Returns:
            instance: Email object.

        """
        data = response['data']
        email = Email()
        email.set_body(data['body'])
        email.set_gateways_associated(data['gateways_associated'])
        email.set_error_list(data['error_list'])
        email.set_subject(data['subject'])
        email.set_attach_pdf(data['attach_pdf'])
        email.set_file_name(data['file_name'])
        for value in data['from_emails']:
            from_email = FromEmail()
            from_email.set_user_name(value['user_name'])
            from_email.set_selected(value['selected'])
            from_email.set_email(value['email'])
            from_email.set_is_org_email_id(value['is_org_email_id'])
            email.set_from_emails(from_email)
        email.set_file_name_without_extension(data[\
        'file_name_without_extension'])
        email.set_deprecated_placeholders_used(data[\
        'deprecated_placeholders_used'])
        email.set_gateways_configured(data['gateways_configured'])
        for value in data['to_contacts']:
            to_contact = ToContact()
            to_contact.set_first_name(value['first_name'])
            to_contact.set_selected(value['selected'])
            to_contact.set_phone(value['phone'])
            to_contact.set_email(value['email'])
            to_contact.set_last_name(value['last_name'])
            to_contact.set_salutation(value['salutation'])
            to_contact.set_contact_person_id(value['contact_person_id'])
            to_contact.set_mobile(value['mobile'])
            email.set_to_contacts(to_contact)
        email.set_attachment_name(data['attachment_name'])
        email.set_customer_id(data['customer_id'])
        return email
   
    def get_billing_address(self,response):
        """This method parses the given response and returns address object for 
            billing address.

        Args:
            response(dict): Resonse containing json object for billing address.
 
        Returns:
            instance: Address object for billing address.
 
        """
        address = response['billing_address']
        address_object = Address()
        address_object.set_address(address['address'])
        address_object.set_city(address['city'])
        address_object.set_state(address['state'])
        address_object.set_zip(address['zip'])
        address_object.set_country(address['country'])
        address_object.set_fax(address['fax'])
        address_object.set_is_update_customer(address['is_update_customer'])
        return address_object

    def get_shipping_address(self,response):
        """This method parses the given response and returns address object 
            for shipping address.
     
        Args:
            response(dict): Response containing json object for shipping 
                address.

        Returns:
            instance: Address object for shipping address.

        """ 
        address = response['shipping_address']
        address_object = Address()
        address_object.set_address(address['address'])
        address_object.set_city(address['city'])
        address_object.set_state(address['state'])
        address_object.set_zip(address['zip'])
        address_object.set_country(address['country'])
        address_object.set_fax(address['fax'])
        address_object.set_is_update_customer(address['is_update_customer'])
        return address_object

    def invoice_template_list(self,response):
        """This method parses the given response and returns list of templates 
            object.

        Args: 
            response(dict): Response containing json object for templates list.

        Returns: 
            list of instances: list of templates object.

        """
        templates = TemplateList()
        for value in response['templates']:
            template = Template()
            template.set_template_name(value['template_name'])
            template.set_template_id(value['template_id'])
            template.set_template_type(value['template_type'])
            templates.set_templates(template)
        return templates

    def payments_list(self,response):
        """This method parses the given response and returns list of invoice 
            payments.

        Args:
            response(dict): Response containing json object for invoice 
                payments list.

        Returns: 
            lsit of instances: List of invoice payments object.

        """
        payments = PaymentList()
        for value in response['payments']:
            payment = InvoicePayment()
            payment.set_payment_id(value['payment_id'])
            payment.set_payment_number(value['payment_number'])
            payment.set_invoice_id(value['invoice_id'])
            payment.set_invoice_payment_id(value['invoice_payment_id'])
            payment.set_payment_mode(value['payment_mode'])
            payment.set_description(value['description'])
            payment.set_date(value['date'])
            payment.set_reference_number(value['reference_number'])
            payment.set_exchange_rate(value['exchange_rate'])
            payment.set_amount(value['amount'])
            payment.set_tax_amount_withheld(value['tax_amount_withheld'])
            payment.set_is_single_invoice_payment(value[\
            'is_single_invoice_payment'])
            payments.set_payments(payment)
        return payments

    def credits_list(self,response):
        """This method parses the given repsonse and returns list of credits.

        Args: 
            response(dict): Response containing json object for credits.

        Returns:
            instance: Invoice credited list object.

        """
        credits = InvoiceCreditedList()
        for value in response['credits']:
            credit = InvoiceCredited()
            credit.set_creditnote_id(value['creditnote_id'])
            credit.set_creditnotes_invoice_id(value['creditnotes_invoice_id'])
            credit.set_credited_date(value['credited_date'])
            credit.set_amount_applied(value['amount_applied'])
            credits.set_invoices_credited(credit)
        return credits

    def apply_credits(self,response):
        """This method parses the given response and returns Payments and 
            credits object.

        Args:
            response(dict): Response containing json object for apply credits.

        Returns:
            instance: Payments and credits object.

        """
        use_credits = response['use_credits']
        payments_and_credits = PaymentAndCredit()
        for value in use_credits['invoice_payments']:
            invoice_payments = InvoicePayment()
            invoice_payments.set_invoice_payment_id(value[\
            'invoice_payment_id'])
            invoice_payments.set_payment_id(value['payment_id'])
            invoice_payments.set_invoice_id(value['invoice_id'])
            invoice_payments.set_amount_used(value['amount_used'])
            payments_and_credits.set_payments(invoice_payments)
        for value in use_credits['apply_creditnotes']:
            credits = InvoiceCredited()
            credits.set_creditnotes_invoice_id(value['creditnotes_invoice_id'])
            credits.set_creditnote_id(value['creditnote_id'])
            credits.set_invoice_id(value['invoice_id'])
            credits.set_amount_applied(value['amount_applied'])
            payments_and_credits.set_apply_creditnotes(credits)
        return payments_and_credits

    def comments_list(self,response):
        """This method parses the given response and returns list of comments.

        Args:
            response(dict): Response containing json object for comments list.

        Returns:
            instance: Comments list object.

        """ 
        comments = CommentList()
        for value in response['comments']:
            comment = Comment()
            comment.set_comment_id(value['comment_id'])
            comment.set_invoice_id(value['invoice_id'])
            comment.set_description(value['description'])
            comment.set_commented_by_id(value['commented_by_id'])
            comment.set_commented_by(value['commented_by'])
            comment.set_comment_type(value['comment_type'])
            comment.set_operation_type(value['operation_type'])
            comment.set_date(value['date'])
            comment.set_date_description(value['date_description'])
            comment.set_time(value['time'])
            comment.set_transaction_id(value['transaction_id'])
            comment.set_transaction_type(value['transaction_type'])
            comments.set_comments(comment)
        return comments

    def get_comment(self,response):
        """This method parses the given response and returns comments object.

        Args:
            response(dict): Response containing json object for comments.

        Returns:
            instance: Comments object.

        """
        comment = response['comment']
        comment_obj = Comment()
        comment_obj.set_comment_id(comment['comment_id'])
        comment_obj.set_invoice_id(comment['invoice_id'])
        comment_obj.set_description(comment['description'])
        comment_obj.set_commented_by_id(comment['commented_by_id'])
        comment_obj.set_commented_by(comment['commented_by'])
        comment_obj.set_date(comment['date'])
        comment_obj.set_date_description(comment['date_description'])
        comment_obj.set_time(comment['time'])
        comment_obj.set_comment_type(comment['comment_type'])
        return comment_obj 
    
     
     
      
     
        
   



#$Id$

class VendorPayment:
    """This class is used to create object for Vendor payments."""
    def __init__(self):
        """Initialize parameters for Vendor payments."""
        self.payment_id = ''
        self.vendor_id = ''
        self.vendor_name = ''
        self.payment_mode = ''
        self.description = ''
        self.date = ''
        self.reference_number = ''
        self.exchange_rate = 0.0
        self.amount = 0.0
        self.currency_symbol = ''
        self.paid_through_account_id = ''
        self.paid_through_account_name = ''
        self.bills = []
        self.balance = 0.0
  
    def set_payment_id(self, payment_id):
        """Set payment id.

        Args:
            payment_id(str): Payment id.

        """
        self.payment_id = payment_id
 
    def get_payment_id(self):
        """Get payment id.

        Returns:
            str: Payment id.

        """
        return self.payment_id
  
    def set_vendor_id(self, vendor_id):
        """Set vendor id.
 
        Args:
            vendor_id(str): Vendor id.

        """
        self.vendor_id = vendor_id

    def get_vendor_id(self): 
        """Get vendor id.

        Returns:
            str: Vendor id.

        """
        return self.vendor_id

    def set_vendor_name(self, vendor_name):
        """Set vendor name.

        Args:
            vendor_name(str): Vendor name.

        """
        self.vendor_name = vendor_name
 
    def get_vendor_name(self):  
        """Get vendor name.

        Returns:
            str: Vendor name.

        """
        return self.vendor_name
 
    def set_payment_mode(self, payment_mode):
        """Set payment mode.

        Args:
            payment_mode(str): Payment mode.

        """
        self.payment_mode = payment_mode

    def get_payment_mode(self):
        """Get payment mode.

        Returns:
            str: Payment mode.

        """
        return self.payment_mode
   
    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self): 
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_date(self, date): 
        """Set date.

        Args: 
            date(str): Date.

        """
        self.date = date

    def get_date(self): 
        """Get date.

        Returns: 
            str: Date.

        """
        return self.date
 
    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_exchange_rate(self, exchange_rate): 
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self): 
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate
  
    def set_amount(self, amount):
        """Set amount.

        Args:
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self):
        """Get amount.

        Returns:
            float: Amount.

        """
        return self.amount

    def set_currency_symbol(self, currency_symbol):
        """Set currency symbol.

        Args:
            currency_symbol(str): Currency symbol.

        """
        self.currency_symbol = currency_symbol

    def get_currency_symbol(self):
        """Get currency symbol.

        Returns:
            str: Currency symbol.

        """
        return self.currency_symbol

    def set_paid_through_account_id(self, paid_through_account_id):
        """Set paid through account id.

        Args:
            paid_through_account_id(str): Paid through account id.

        """
        self.paid_through_account_id = paid_through_account_id

    def get_paid_through_account_id(self):
        """Get paid through account id.

        Returns:
            str: Paid through acount id.

        """
        return self.paid_through_account_id
  
    def set_paid_through_account_name(self, paid_through_account_name):
        """Set paid through account name.

        Args:
            paid_through_account_name(str): Paid through account name.

        """
        self.paid_through_account_name = paid_through_account_name

    def get_paid_through_account_name(self):
        """Get paid through account name.
 
        Returns:
            str: Paid through account name.

        """
        return self.paid_through_account_name
  
    def set_bills(self, bill): 
        """Set bills.

        Args:
            bills(instance): Bills object.

        """
        self.bills.append(bill)

    def get_bills(self):
        """Get bills.

        Returns:
            list of instance: List of bills object.

        """
        return self.bills
    
    def set_balance(self, balance):
        """Set balance.

        Args: 
            balance(float): Balance.

        """
        self.balance = balance

    def get_balance(self):
        """Get balance.

        Returns:
            float: Balance.

        """
        return self.balance

    def to_json(self):
        """This method is used to convert vendor payments object to json object.

        Returns:
            dict: Dictionary containing json object for vendor payments.

        """ 
        data = {}
        if self.vendor_id != '':
            data['vendor_id'] = self.vendor_id
        if self.bills:
            data['bills'] = []
            for value in self.bills:
                bill = value.to_json()
                data['bills'].append(bill)
        if self.payment_mode != '':
            data['payment_mode'] = self.payment_mode
        if self.description != '': 
            data['description'] = self.description
        if self.date != '':
            data['date'] = self.date
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.amount > 0:
            data['amount'] = self.amount
        if self.paid_through_account_id != '':
           data['paid_through_account_id'] = self.paid_through_account_id
        return data


